--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       ctrller.vhd
--
--  Description :     
--                    Main DDR SDRAM controller block. This includes the following
--                    features:
--                    - The controller state machine that controls the 
--                    initialization process upon power up, as well as the 
--                    read, write, and refresh commands. 
--                    - Accepts and decodes the user commands.
--                    - Generates the address and Bank address signals
--                    - Generates control signals for other modules, including
--                    the control signals for the dqs_en block.
--
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity controller is
port(
    clk180           : in std_logic;
    dcm_lock         : in std_logic;
    clk_dv16         : in std_logic;
    rst              : in std_logic;
    read_address     : in std_logic_vector(21 downto 0);
    write_address     : in std_logic_vector(21 downto 0);
    config_register  : in std_logic_vector(9 downto 0);
    command_register : in std_logic_vector(2 downto 0);
    wrdata_fifo_empty : in std_logic;
    wraddr_fifo_empty : in std_logic;
    rdaddr_fifo_empty : in std_logic;
    controller_wrdata_fifo_rden : out std_logic;
    controller_wraddr_fifo_rden : out std_logic;
    controller_rdaddr_fifo_rden : out std_logic;
    burst_length_val : out std_logic_vector(2 downto 0);
    ddr_rasb         : out std_logic;
    ddr_casb         : out std_logic;
    ddr_web          : out std_logic;
    ddr_BA           : out std_logic_vector(1 downto 0);
    ddr_address      : out std_logic_vector(11 downto 0);
    ddr_cke          : out std_logic;
    ddr_csb          : out std_logic;
    dqs_enable       : out std_logic;
    dqs_reset        : out std_logic;
    write_enable     : out std_logic;
    read_enable      : out std_logic;
    cntrl_data_val   : out std_logic);
end controller;


architecture arc_controller of controller is

type s_m is (IDLE, PRECHARGE, LOAD_MODE_REG, AUTO_REFRESH, ACTIVE, 
         FIRST_WRITE, WRITE_WAIT, BURST_WRITE, WRITE_AFTER_READ, READ_AFTER_WRITE, precharge_after_rd_wr,
         READ_WAIT, FIRST_READ, BURST_READ, BURST_STOP);
signal next_state,current_state: s_m;



signal address_config         : std_logic_vector(11 downto 0);
signal AUTO_REF               : std_logic;
signal AUTO_REF1               : std_logic;
signal AUTO_REF2               : std_logic;
signal AUTOREF_value          : std_logic;
signal AUTO_REF_detect        : std_logic;
signal AUTO_REF_pulse_end     : std_logic;
signal AUTOREF_COUNT          : std_logic_vector(7 downto 0);
signal Burst_terminate        : std_logic;
signal bank_address_active      : std_logic_vector(1 downto 0);
signal bank_address           : std_logic_vector(1 downto 0);
signal bank_address_set       : std_logic;
signal bank_address_set_1       : std_logic;
signal bank_address_conflict    : std_logic;
signal bank_address_reg         : std_logic_vector(1 downto 0);
signal burst_length           : std_logic_vector(2 downto 0);
signal burst_cnt_max          : std_logic_vector(2 downto 0);
signal CAS_COUNT              : std_logic_vector(1 downto 0);
signal CAS_COUNT_reg          : std_logic_vector(1 downto 0);
signal CAS_COUNT_reg1          : std_logic_vector(1 downto 0);
signal CAS_COUNT_check        : std_logic;
signal cas_count_value        : std_logic_vector(1 downto 0);
signal cas_b                  : std_logic;
signal cas_latency            : std_logic_vector(2 downto 0);
signal COLUMN_ADDRESS_reg     : std_logic_vector(7 downto 0);
signal COLUMN_ADDRESS_reg1     : std_logic_vector(7 downto 0);
signal COLUMN_ADDRESS_reg2     : std_logic_vector(7 downto 0);
signal COLUMN_ADDRESS_reg3     : std_logic_vector(7 downto 0);
signal COLUMN_ADDRESS_reg4     : std_logic_vector(7 downto 0); 
signal COLUMN_ADDRESS_reg5     : std_logic_vector(7 downto 0);
signal COLUMN_ADDRESS_reg6     : std_logic_vector(7 downto 0);
signal COLUMN_ADDRESS         : std_logic_vector(7 downto 0);
signal command_reg            : std_logic_vector(2 downto 0);
signal config_reg             : std_logic_vector(9 downto 0);
signal CONFLICT               : std_logic;
signal CONFLICT_value         : std_logic;
signal controller_wraddr_fifo_rden_r : std_logic;
signal controller_wraddr_fifo_rden_2r : std_logic;
signal controller_rdaddr_fifo_rden_r : std_logic;
signal controller_rdaddr_fifo_rden_2r : std_logic;  
signal controller_rdaddr_fifo_rden_3r : std_logic;
signal controller_rdaddr_fifo_rden_4r : std_logic;
signal controller_wrdata_fifo_rden_r : std_logic;
signal controller_wrdata_fifo_rden_2r : std_logic;
signal controller_wraddr_fifo_rden_3r : std_logic;
signal cntrl_data_val1        : std_logic;
signal cntrl_data_val2        : std_logic ;
signal data_val               : std_logic;
signal ddr_rasb1              : std_logic;
signal ddr_casb1              : std_logic;
signal ddr_web1               : std_logic;
signal ddr_rasb2              : std_logic;
signal ddr_casb2              : std_logic;
signal ddr_web2               : std_logic;
signal ddr_rasb3              : std_logic;
signal ddr_casb3              : std_logic;
signal ddr_web3               : std_logic;
signal ddr_rasb4              : std_logic;
signal ddr_casb4              : std_logic;
signal ddr_web4               : std_logic;
signal ddr_ba1                : std_logic_vector(1 downto 0);
signal ddr_ba2                : std_logic_vector(1 downto 0);
signal ddr_ba3                : std_logic_vector(1 downto 0);
signal ddr_ba4                : std_logic_vector(1 downto 0);
signal ddr_ba5                : std_logic_vector(1 downto 0);
signal ddr_address1           : std_logic_vector(11 downto 0);
signal ddr_address2           : std_logic_vector(11 downto 0);
signal ddr_address3           : std_logic_vector(11 downto 0);
signal ddr_address4           : std_logic_vector(11 downto 0);
signal DQS_enable_out         : std_logic;
signal DQS_reset_out          : std_logic;
signal INIT_COUNT_value       : std_logic_vector(2 downto 0);
signal INIT_COUNT             : std_logic_vector(2 downto 0);
signal INIT_MEMORY            : std_logic;
signal ld_mode                : std_logic;
signal load_cmd               : std_logic;
signal LOAD_RCD_read          : std_logic;
signal LOAD_RCD_write         : std_logic;
signal MRD_COUNT_value        : std_logic_vector(1 downto 0);
signal max_ref_cnt            : std_logic_vector(7 downto 0);
signal MRD_COUNT              : std_logic_vector(1 downto 0);
signal PRECHARGE_cmd          : std_logic;
signal ras_count_value        : std_logic_vector(3 downto 0);
signal ras_count              : std_logic_vector(3 downto 0);
signal rdburst_end_cnt1       : std_logic_vector(2 downto 0);
signal rdburst_end_cnt2       : std_logic_vector(2 downto 0);
signal rdburst_end_cnt_chk    : std_logic_vector(2 downto 0);
signal read_cmd1              : std_logic;
signal read_cmd2              : std_logic;
signal RRD_COUNT              : std_logic_vector(1 downto 0);
signal RCDR_COUNT             : std_logic_vector(2 downto 0);
signal RCDW_COUNT             : std_logic_vector(1 downto 0);
signal RP_cnt_value           : std_logic_vector(2 downto 0);
signal RFC_COUNT_value        : std_logic_vector(3 downto 0);
signal RRD_COUNT_value        : std_logic_vector(1 downto 0);
signal RCDR_COUNT_value       : std_logic_vector(2 downto 0);
signal RCDW_COUNT_value       : std_logic_vector(1 downto 0);
signal RC_COUNT_value         : std_logic_vector(3 downto 0);
signal rdburst_end_cnt_value  : std_logic_vector(2 downto 0);
signal rdburst_end_cnt        : std_logic_vector(2 downto 0);
signal rdburst_end            : std_logic;
signal RP_COUNT               : std_logic_vector(2 downto 0);
signal RC_COUNT               : std_logic_vector(3 downto 0);
signal RFC_COUNT              : std_logic_vector(3 downto 0);
signal Read_enable_out        : std_logic;
signal read_cmd_in            : std_logic;
signal read_address_reg       : std_logic_vector(21 downto 0); 
signal ROW_ADDRESS            : std_logic_vector(11 downto 0);
signal ROW_ADDRESS_reg        : std_logic_vector(11 downto 0);
signal row_address_active     : std_logic_vector(11 downto 0) := "000000000000";
signal row_address_active_reg : std_logic_vector(11 downto 0);
signal row_address_conflict   : std_logic;
signal row_address_set        : std_logic;
signal row_address_set_1        : std_logic;
signal setup_wait             : std_logic_vector(2 downto 0);
signal wait_cycles            : std_logic_vector(3 downto 0);
signal wrburst_end_cnt_value  : std_logic_vector(2 downto 0);
signal wrburst_end_cnt        : std_logic_vector(2 downto 0);
signal wrburst_end            : std_logic;
signal wrburst_chk            : std_logic;
signal wrburst_chk_value      : std_logic;
signal we_b                   : std_logic;
signal WR_COUNT               : std_logic_vector(1 downto 0);
signal WR_COUNT_value         : std_logic_vector(1 downto 0);
signal write_address_reg      : std_logic_vector(21 downto 0); 
signal Write_enable_out       : std_logic;
signal Write_cmd1             : std_logic;
signal Write_cmd2             : std_logic;
signal Write_cmd              : std_logic;
signal write_cmd_in           : std_logic;
signal write_cmd_issued       : std_logic;  
signal wraddr_fifo_count      : std_logic_vector(2 downto 0);
signal wrdata_fifo_empty_reg  : std_logic;
signal wraddr_fifo_empty_reg  : std_logic;
signal rdaddr_fifo_empty_reg  : std_logic;   

begin

--  Input : CONFIG REGISTER FORMAT 
-- config_register = {   EMR(Enable/Disable DLL),
--                       BMR (Normal operation/Normal Operation with Reset DLL),
--                       BMR/EMR,
--                       CAS_latency (3),
--                       Burst type ,
--                       Burst_length (3) }
--
-- Input : COMMAND REGISTER FORMAT
--          
--          101  - Load Mode Register
--          111  - Burst terminate
--          others - NOP
--
-- Input : Address format
--   row address = input address(19 downto 8)
--   column addrs = input address( 7 downto 0)
--

ddr_csb <= '0';
ddr_cke <= '1';
ROW_ADDRESS                 <= write_address(21 downto 10) when controller_wraddr_fifo_rden_r = '1' else
                               read_address(21 downto 10) when controller_rdaddr_fifo_rden_2r = '1' else
                               row_address_reg; 
COLUMN_ADDRESS              <= write_address(9 downto 2) when controller_wraddr_fifo_rden_r = '1' else
                               read_address(9 downto 2) when controller_rdaddr_fifo_rden_2r = '1' else
                               column_address_reg;                               
bank_ADDRESS                <= write_address(1 downto 0) when controller_wraddr_fifo_rden_r = '1' else
                               read_address(1 downto 0) when controller_rdaddr_fifo_rden_2r = '1' else
                               bank_address_reg; 

burst_length_val            <= burst_length;
controller_wraddr_fifo_rden <= controller_wraddr_fifo_rden_r;
controller_rdaddr_fifo_rden <= controller_rdaddr_fifo_rden_r;
controller_wrdata_fifo_rden <= controller_wrdata_fifo_rden_r;


--register input commands from the user
process(clk180, rst)
  begin
    if rst = '1' then
        config_reg <= "0000000000";
        command_reg <= "000";
        ROW_ADDRESS_reg <= "000000000000";
        COLUMN_ADDRESS_reg <= "00000000";
        bank_address_reg <= "00";
        read_address_reg <= "0000000000000000000000";
        write_address_reg <= "0000000000000000000000";
    elsif clk180'event and clk180 = '1' then
        config_reg <= config_register;
        command_reg <= command_register;
        ROW_ADDRESS_reg <= ROW_ADDRESS;
        COLUMN_ADDRESS_reg <= COLUMN_ADDRESS;
        bank_address_reg <= Bank_address;
        read_address_reg <= read_address;
        write_address_reg <= write_address;
    end if;
  end process;
  
  
process(clk180,rst)
begin
  if rst = '1' then   
      wrdata_fifo_empty_reg <= '0';
      wraddr_fifo_empty_reg <= '0';
      rdaddr_fifo_empty_reg <= '0';
  elsif clk180'event and clk180 = '1' then
      wrdata_fifo_empty_reg <= wrdata_fifo_empty;
      wraddr_fifo_empty_reg <=  wraddr_fifo_empty;
      rdaddr_fifo_empty_reg <=  rdaddr_fifo_empty;
  end if;
end process;
     
      
                              
                              
process(clk180, rst)          
begin
  if rst = '1' then
     burst_length <= "000";    
     cas_latency  <= "000";    
  elsif clk180'event and clk180 = '1' then
     burst_length <= config_reg(2 downto 0);    
     cas_latency  <= config_reg(6 downto 4);
  end if;
end process;

read_cmd_in <= '1' when rdaddr_fifo_empty_reg = '0' else
               '0';
               
write_cmd_in <= '1' when wrdata_fifo_empty_reg = '0' else
                '0';

process(clk180, rst)
begin
  if rst = '1' then  
       burst_terminate <= '0';
      load_cmd <= '0';
  elsif clk180'event and clk180 = '1' then
      case command_reg is
         when "101" =>
             load_cmd <= '1';
         when "111" =>
             Burst_terminate <= '1';
         when others =>
             load_cmd <= '0';
             Burst_terminate <= '0';
      end case;
  end if;
end process;

--**********************************************************************************************
-- Burst count value counter when there are consecutive READs or WRITEs
-- While doing consecutive READs or WRITEs, the Burst_count value determines when the next 
-- READ or WRITE command should be issued. The burst length is determined while loading the 
-- Load Mode Register
-- burst_cnt_max shows the number of clock cycles for each butst
-- burst_cnt_max = 1 for a burst length of 2, since it is ddr
--**********************************************************************************************
burst_cnt_max <= "001" when burst_length = "001" else
                 "010" when burst_length = "010" else
                 "100" when burst_length = "011" else
                 "111" when burst_length = "111" else
                 "000";

--  to initialize memory

process(clk180, rst)
begin
  if rst = '1' then
     INIT_MEMORY <= '1';
  elsif clk180'event and clk180 = '1' then
    if INIT_COUNT = "111" and MRD_COUNT = "00" then
     INIT_MEMORY <= '0';
    else
     INIT_MEMORY <= INIT_MEMORY;
    end if;
  end if;
end process;

-- counter for Memory Initialization sequence
INIT_COUNT_value <= (INIT_COUNT + "001") when ((next_state = PRECHARGE) or (next_state = LOAD_MODE_REG)
                                  or (next_state = AUTO_REFRESH)) and INIT_MEMORY = '1' else
                    INIT_COUNT ;

--*******************************************************************************************
-- INTERNAL COUNTERS
--*******************************************************************************************

-- Register counter values

process(clk180, rst)
begin
 if rst = '1'  then

     INIT_COUNT     <= "000";
     RP_COUNT       <= "000";
     MRD_COUNT      <= "00";
     RFC_COUNT      <= "0000";
     RAS_COUNT      <= "0000";
     CAS_COUNT      <= "00";
     CAS_COUNT_reg  <= "00";
     CAS_COUNT_reg1 <= "00";
     RRD_COUNT      <= "00";
     RCDR_COUNT     <= "000";
     RCDW_COUNT     <= "00";
     RC_COUNT       <= "0000";
     RDBURST_END_CNT <= "000";
     RDBURST_END_CNT1 <= "000";
     RDBURST_END_CNT2 <= "000";
     RDBURST_END_CNT_CHK <= "000";
     WRburst_end_cnt <= "000";  
     WR_COUNT        <= "00";
      
  elsif clk180'event and clk180 = '1' then

     INIT_COUNT     <= INIT_COUNT_value;
     RP_COUNT       <= RP_CNT_value;
     MRD_COUNT      <= MRD_COUNT_value;
     RFC_COUNT      <= RFC_COUNT_value;
     RAS_COUNT      <= RAS_COUNT_value;
     CAS_COUNT      <= CAS_COUNT_value;
     CAS_COUNT_reg  <= CAS_COUNT_reg1;
     CAS_COUNT_reg1  <= CAS_COUNT;
     RRD_COUNT      <= RRD_COUNT_value;
     RCDR_COUNT     <= RCDR_COUNT_value;
     RCDW_COUNT     <= RCDW_COUNT_value;
     RC_COUNT       <= RC_COUNT_value;
     WRBURST_END_CNT <= WRBURST_END_CNT_value;
     RDBURST_END_CNT1 <= RDBURST_END_CNT_value;
     RDBURST_END_CNT2 <= RDBURST_END_CNT1;
     RDBURST_END_CNT <= RDBURST_END_CNT2;
     RDBURST_END_CNT_CHK <= RDBURST_END_CNT_value;
     WR_COUNT        <= WR_COUNT_value;
     
  end if;
end process;

--********************************************************************************************                
-- MRD Counter
-- an executable command can be issued only after Tmrd(2 cycles) after a LMR command is issued
--********************************************************************************************
MRD_COUNT_value <= "10" when (next_state = LOAD_MODE_REG) else
                   (MRD_COUNT - "01") when (MRD_COUNT /= "00") else
                    MRD_COUNT;

--********************************************************************************************                
-- RFC Counter
-- an executable command can be issued only after Trfc(60 ns => 60/5 = 12 cycles) 
--                                after a AUTOREFRESH command is issued
--********************************************************************************************
RFC_COUNT_value <= "1111" when (next_state = AUTO_REFRESH) else
             (RFC_COUNT - "0001") when (RFC_COUNT /= "0000") else
              RFC_COUNT;

--********************************************************************************************                
-- RP Counter
-- an executable command can be issued only after Trp(20 ns for a -5 device => 4 cycles) 
--                        after a PRECHARGE command is issued
--********************************************************************************************
RP_cnt_value <= "100" when (next_state = PRECHARGE ) else
                (RP_COUNT - "001") when (RP_COUNT /= "000") else
                 RP_COUNT;

--********************************************************************************************                
-- RRD Counter
-- minimum interval between successive ACTIVE commands to different banks - Trrd
-- 2 clock cycles
--********************************************************************************************
RRD_COUNT_value <= "10" when (next_state = ACTIVE) else
                   (RRD_COUNT - 1) when RRD_COUNT /= "00" else
                    "00"; 

--*********************************************************************************************
-- ACTIVE to READ/WRITE counter
-- RCDr counter
-- ACTIVE to READ delay - (-5) device requires 20 ns of delay => 4 clock cycles
--
-- RCDW counter
-- ACTIVE to WRITE delay - (-5) device requires 10 ns of delay => 2 clock cycles
--
--*********************************************************************************************
RCDR_COUNT_value <=  "100" when ( next_state = ACTIVE) else
                   (RCDR_COUNT - "001" ) when (RCDR_COUNT /= "000") else
                    RCDR_COUNT;

RCDW_COUNT_value <= "11" when  (next_state = ACTIVE and burst_length = "001") else
                    "10" when (next_state = ACTIVE and burst_length(1) = '1') else
                   (RCDW_COUNT - "01") when (RCDW_COUNT /= "00") else
                    RCDW_COUNT;

--*********************************************************************************************
-- ACTIVE to PRECHARGE counter
-- RAS counter
-- ACTIVE to PRECHARGE delay -
-- the memory device require 40 ns (8 clock cycles)after issuing an ACTIVE command before issuing a
-- PRECHARGE command
--
--*********************************************************************************************
RAS_count_value <= "1000" when next_state = ACTIVE else
                    RAS_COUNT - "0001" when RAS_COUNT /= "0000" else
                    RAS_COUNT;

--**********************************************************************************************
-- RC counter
-- an ACTIVE command to a different row can be issued only after the previous row has been 
-- precharged & after Trc is met 
-- Trc = 60 ns = 12 clock cycles
--**********************************************************************************************
RC_COUNT_value <= "1100" when next_state = ACTIVE else
                  (RC_COUNT - "0001") when (RC_COUNT /= "0000") else
                   RC_COUNT;


--********************************************************************************************                
-- WR Counter
-- a PRECHARGE command can be applied only after 2 cycles after a WRITE command has finished 
-- executing
--********************************************************************************************
WR_COUNT_value <= "10" when (next_state = precharge_after_rd_wr) else
                  (WR_COUNT - "01") when (WR_COUNT /= "00") else
                  WR_COUNT;

--*********************************************************************************************
-- Auto refresh counter - the design uses AUTO REFRESH
-- 
-- the DDR SDRAM requires AUTO REFRESH cycles at an average interval of 15.6 us
-- the minimum frequency obtained from the system clock = 200 MHz/16 = 12.5 MHz = 80 ns
-- Hence, a counter value of 195 should be used to obtain a 15.6 us clock for Auto Refresh
-- Binary value of 195 = 11000011
--*********************************************************************************************
max_ref_cnt <= "11000011";

AUTOREF_value  <= '1' when (AUTOREF_COUNT = max_ref_cnt) else
                  '0';


-- counter to count upto 195 (binary - 11000011)
process(clk_dv16, rst)
begin
if rst = '1'  then
  AUTOREF_COUNT <= "00000000";
  AUTO_REF_detect <= '0';
elsif clk_dv16'event and clk_dv16 = '1' then
  if AUTOREF_COUNT = max_ref_cnt then
     AUTOREF_COUNT <= "00000000";
  else
     AUTOREF_COUNT <= AUTOREF_COUNT + "00000001";
  end if;
  AUTO_REF_detect <= AUTOREF_value;
end if;
end process;

process(clk180, rst)
begin
  if rst = '1'  then
      AUTO_REF_pulse_end <= '0';
      --AUTO_REF_detect <= '0';
  elsif clk180'event and clk180 = '1' then
      --AUTO_REF_detect <= AUTOREF_value;
      AUTO_REF_pulse_end <= AUTO_REF_detect;
  end if;
end process;

process(clk180,rst)
begin
  if rst = '1' then
      AUTO_REF1 <= '0';
      AUTO_REF2 <= '0';
  elsif clk180'event and clk180 = '1' then
      if AUTO_REF_detect = '1' and AUTO_REF_pulse_end /= '1' then
         AUTO_REF1 <= '1';
      else
         AUTO_REF1 <= '0';
      end if;
      AUTO_REF2 <= AUTO_REF1;
  end if;
end process;

--  AUTO_REF signal being high flags the need for auto refresh to be done. This signal is 
--  held high as long as the controller issues an auto refresh command. This signal goes low when the
--  controller enters the AUTO REFRESH state
process(clk180, rst)
begin
  if rst = '1' then
      AUTO_REF    <= '0';
  elsif clk180'event and clk180 = '1' then
      if (AUTO_REF1 = '1') and (AUTO_REF2 = '0') then
         AUTO_REF <= '1';
      elsif next_state = AUTO_REFRESH then 
         AUTO_REF <= '0';
      else
         AUTO_REF <= AUTO_REF;
      end if;
  end if;
end process;

--**************************************************************************
-- WRITE TRANSACTIONS
--**************************************************************************

-- register write_cmd until WRITE command needs to be issued

  process(clk180, rst)
  begin
     if rst = '1' then
       write_cmd1 <= '0';
       write_cmd2 <= '0';
     elsif clk180'event and clk180 = '1' then
       write_cmd1 <= write_cmd_in;
       write_cmd2 <= write_cmd1;
                
     end if;
  end process; 
           
WRburst_end_cnt_value <= burst_cnt_max  when next_state = FIRST_WRITE or next_state = BURST_WRITE else
                         WRburst_end_cnt - "001" when WRburst_end_cnt /= "000" else
                         "000";

-- to check if a WRITE is in progress
wrburst_chk_value <= WRburst_end_cnt(2) or WRburst_end_cnt(1) or WRburst_end_cnt(0);  

-- wrburst_end is generated based on burst length inorder to achieve consecutive bursts
process(clk180, rst)
begin
  if rst = '1' then
     wrburst_end <= '0';
     wrburst_chk <= '0';
  elsif clk180'event and clk180 = '1' then
         if (next_state = FIRST_WRITE or next_state = BURST_WRITE or wrburst_end_cnt(1) = '1') then
           wrburst_end <= '1';
         else
            wrburst_end <= '0';
         end if;
         wrburst_chk <= wrburst_chk_value;
  end if;
end process;   

--************************************************************************
-- READ TRANSACTIONS
--************************************************************************

  process(clk180, rst)
  begin
     if rst = '1' then
       read_cmd1 <= '0';
       read_cmd2 <= '0';
       
     elsif clk180'event and clk180 = '1' then
       read_cmd1 <= read_cmd_in;
       read_cmd2 <= read_cmd1;
       
     end if;
  end process; 
  
 
--********************************************************************************************
-- CAS latency counter
-- CAS latencies of 2,3,4 can be set using Mode register bits M(6:4)
--
--      M6  M5  M4   CAS latency
--      0    1   0 -    2
--      0    1   1 -    3
--      1    0   0 -    4
--      others     -   reserved
-- This design uses a CAS latency of 3 for a clock rate of 200 MHz
--
--********************************************************************************************
cas_count_value <= "11" when (NEXT_STATE = FIRST_READ) else
                    CAS_COUNT - "01" when CAS_COUNT /= "00" else
                    "00";  
                    
RDburst_end_cnt_value <= burst_cnt_max when (NEXT_STATE = FIRST_READ or NEXT_STATE = BURST_READ) else
                      RDBURST_END_CNT1 - "001" when RDBURST_END_CNT1 /= "000" else
                     "000" ;
-- rdburst_end needs to be generated based on burst lengths while doing consecutive bursts
process(clk180, rst)
begin
 if rst = '1' then
    rdburst_end <= '0';
 elsif clk180'event and clk180 = '1' then
    if burst_cnt_max(2) = '0' then
       if (next_state = FIRST_READ or next_state = BURST_READ or cas_count = "11" ) then
         rdburst_end <= '1';
       else
         rdburst_end <= '0';
       end if;
    end if;
 end if;
end process;          
                                 

--**********************************************************************************************
-- to generate the Data Strobe enable and reset signal for the DQS_STROBE module
-- The DQS signal needs to be generated center aligned with the data. 
-- The controller generates the DQS enable signal when the state machine is in the FIRST_WRITE
-- state,to take care of the write preamble
--**********************************************************************************************
DQS_enable_out <= '1' when ( (next_state = FIRST_WRITE) or (next_state = BURST_WRITE)  or -- WRburst_chk = '1') else
                             (WRburst_end_cnt /= "000")  ) else
        
                  '0';

DQS_reset_out  <= '1' when next_state = FIRST_WRITE else
                 '0'; 

process(clk180, rst)
begin
if rst = '1' then
    DQS_enable <= '0';
    DQS_reset  <= '0';
    
elsif clk180'event and clk180 = '1' then
    DQS_enable <= DQS_enable_out;
    DQS_reset  <= DQS_reset_out;
        
end if;
end process;

--****************************************************************************************************
--Generating WRITE and READ enable signals to the data path module
--*****************************************************************************************************
CAS_count_check <= cas_count_reg(1) ; --or cas_count_reg(0);

write_enable_out <= '1' when WRBurst_chk = '1' else
                     '0'; 

read_enable_out <= '1' when ( (rdburst_end_cnt /= "000" ) or (CAS_COUNT_check = '1') ) else
                    '0';

process(clk180, rst)
begin
if rst = '1' then
   Read_enable <= '0'; 
   write_enable <= '0';
elsif clk180'event and clk180 = '1' then
   Read_enable <= Read_enable_out;
   write_enable <= write_enable_out;
end if;
end process;

data_val <= RDBURST_END_CNT(2) or RDBURST_END_CNT(1) or RDBURST_END_CNT(0);

process(clk180,rst)
begin
  if rst = '1' then
      cntrl_data_val1 <= '0';
      cntrl_data_val2 <= '0';
      cntrl_data_val  <= '0';
  elsif clk180'event and clk180 = '1' then
     cntrl_data_val1 <= data_val;
     cntrl_data_val2 <= cntrl_data_val1;
     cntrl_data_val <= cntrl_data_val2;
  end if;
end process;

-- wait cycles for the controller
process(clk180, rst)
begin
  if rst = '1' then
     wait_cycles <= "1100";
  elsif clk180'event and clk180 = '1' then
     if (CURRENT_STATE = WRITE_WAIT or CURRENT_STATE = READ_WAIT) then
        wait_cycles <= wait_cycles - "0001";
     else
        wait_cycles <= "0100";
     end if;
  end if;
end process; 
--*********************************************************************************************
-- FIFO read enable signals
--*********************************************************************************************
-- read enable for the write address fifo

process(clk180, rst)
begin
   if rst = '1' then
       controller_wraddr_fifo_rden_r <= '0';
       controller_wraddr_fifo_rden_2r <= '0';
       controller_wraddr_fifo_rden_3r <= '0';
   elsif clk180'event and clk180 = '1' then
      if (wraddr_fifo_empty = '0' and rdburst_end_cnt = "000") and 
          (bank_address_conflict = '0' and row_address_conflict = '0') then 
          controller_wraddr_fifo_rden_r <= '1';
      else
          controller_wraddr_fifo_rden_r <= '0';
      end if;
      controller_wraddr_fifo_rden_2r <= controller_wraddr_fifo_rden_r ;
      controller_wraddr_fifo_rden_3r <= controller_wraddr_fifo_rden_2r;
   end if;
end process;       

process(clk180, rst)
begin
   if rst = '1' then
      wraddr_fifo_count <= "000";
   elsif clk180'event and clk180 = '1' then
      if  controller_wraddr_fifo_rden_r  = '1' and burst_length = "010" then
          wraddr_fifo_count <= "001";
      elsif wraddr_fifo_count /= "000" then
          wraddr_fifo_count <= wraddr_fifo_count - "001";
      else
          wraddr_fifo_count <= "000";
      end if ;
   end if;
end process;

-- read enable for the write data fifo
process(clk180, rst)
begin
   if rst = '1' then
       controller_wrdata_fifo_rden_r <= '0';
       controller_wrdata_fifo_rden_2r <= controller_wrdata_fifo_rden_r;
   elsif clk180'event and clk180 = '1' then
        if  (wrdata_fifo_empty = '0' and 
            ( ((NEXT_STATE = FIRST_WRITE or NEXT_STATE = BURST_WRITE or NEXT_STATE = WRITE_WAIT or NEXT_STATE = WRITE_AFTER_READ)
                and (bank_address_conflict = '0' and row_address_conflict = '0')  )
                   or (NEXT_STATE = IDLE and rcdw_count = "10"))) then                                                      
             controller_wrdata_fifo_rden_r <= '1';                                                  
        elsif wrdata_fifo_empty = '1' then 
            controller_wrdata_fifo_rden_r <= '0'; 
         else
            controller_wrdata_fifo_rden_r <= controller_wrdata_fifo_rden_r; 
        end if;                    
     controller_wrdata_fifo_rden_2r <= controller_wrdata_fifo_rden_r;                            
   end if;                                                                                    
end process;

process(clk180, rst)
begin
   if rst = '1' then
       controller_rdaddr_fifo_rden_r <= '0';
       controller_rdaddr_fifo_rden_2r <= '0';   
       controller_rdaddr_fifo_rden_3r <= '0';
       controller_rdaddr_fifo_rden_4r <= '0';
       
   elsif clk180'event and clk180 = '1' then
       if(rdaddr_fifo_empty = '0' and rp_count = "00" and rfc_count = "0000" and 
              (bank_address_conflict = '0' and row_address_conflict = '0')
               and ( NEXT_STATE = FIRST_READ or NEXT_STATE = BURST_READ or NEXT_STATE = READ_AFTER_WRITE 
                   or (NEXT_STATE = READ_WAIT and burst_length = "001")
                   or (NEXT_STATE = IDLE and rcdr_count = "0001")) ) then
           controller_rdaddr_fifo_rden_r <= '1';
       else
           controller_rdaddr_fifo_rden_r <= '0';
       end if;
       controller_rdaddr_fifo_rden_2r <= controller_rdaddr_fifo_rden_r;
       controller_rdaddr_fifo_rden_3r <= controller_rdaddr_fifo_rden_2r;
       controller_rdaddr_fifo_rden_4r <= controller_rdaddr_fifo_rden_3r;
   end if;
end process;

--*********************************************************************************************
-- to check current state for the address bus
--*********************************************************************************************
process(clk180, rst)
begin
if rst = '1'  then
   current_state <= IDLE;
elsif clk180'event and clk180 = '1' then
   current_state <= next_state;
end if;
end process;

--*********************************************************************************************
-- main state machine
--*********************************************************************************************
process(clk180, rst, RP_COUNT, INIT_COUNT, MRD_COUNT, RFC_COUNT, CAS_COUNT)
begin
if rst = '1' then
    next_state <= IDLE;
elsif clk180'event and clk180 = '1' then
    case next_state is
         when IDLE =>
               if INIT_MEMORY = '1' then
                  case INIT_COUNT is 

                   -- this state is for NOP/Deselect
                   when "000" =>
                        
                           next_state <= PRECHARGE;
                        
                   when "001" =>
                         if RP_COUNT = "001" then    -- wait for 4 clock cycles (Trp)
                               next_state <= LOAD_MODE_REG;
                         else
                               next_state <= IDLE;
                         end if;
                   when "010" =>
                         if MRD_COUNT = "01" then
                            next_state <= LOAD_MODE_REG;  -- for reseting DLL in Base Mode register
                         else
                            next_state <= IDLE;
                         end if;
                   when "011" =>
                         if MRD_COUNT = "01" then
                            next_state <= PRECHARGE;
                         else
                            next_state <= IDLE;
                         end if;
                   when "100" => 
                         if RP_COUNT = "001" then -- wait for 4 clock cycles (Trp)
                               next_state <= AUTO_REFRESH;
                         else
                               next_state <= IDLE;
                         end if;
                   when "101" =>
                         if RFC_COUNT = "0001" then
                           next_state <= AUTO_REFRESH;
                         else 
                           next_state <= IDLE;
                         end if;
                   when "110" =>
                         if RFC_count = "0001" then
                            next_state <= LOAD_MODE_REG;  -- to deactivate the rst DLL bit in the LMR
                         else
                            next_state <= IDLE;
                         end if;
                   when others =>
                        next_state <= IDLE;
                   end case; 

            elsif AUTO_REF = '1' and (RP_COUNT = "001" ) then
                  next_state <= AUTO_REFRESH; -- normal Refresh in the IDLE state

            -- check for a read/write command and if no auto refresh in progress
            elsif (load_cmd = '1') then
                  next_state <= LOAD_MODE_REG;

            elsif ((controller_wraddr_fifo_rden_3r = '1' or controller_rdaddr_fifo_rden_2r = '1')  
                   and rcdw_count = "00" and rcdr_count = "000" ) or 
                   ((bank_address_conflict = '1' or row_address_conflict = '1') and 
                      (rp_count = "001" or rfc_count = "0001")) then    
                  next_state <= ACTIVE;       
            
            elsif (  RCDW_COUNT = "01" and rp_count = "000" and 
                     (wrdata_fifo_empty = '0') and 
                     (bank_address_conflict = '0' and row_address_conflict = '0')             ) then
                  next_state <= FIRST_WRITE;

            elsif (RCDR_COUNT = "000" and controller_rdaddr_fifo_rden_2r = '1' and rp_count = "000" ) then
                  next_state <= FIRST_READ;

            else
                  next_state <= IDLE;
            end if;              


         when PRECHARGE =>
              next_state <= IDLE;

         when LOAD_MODE_REG =>
              next_state <= IDLE;
      
         when AUTO_REFRESH =>
              next_state <= IDLE; 
         
         when ACTIVE =>
              next_state <= IDLE;
         
         when FIRST_WRITE =>
              -- to meet the write preamble   
 
              if burst_length = "001" then
                   if (wrdata_fifo_empty = '0' and controller_wrdata_fifo_rden_r = '1')  then 
                       next_state <= BURST_WRITE;
                    else
                       next_state <= WRITE_WAIT;
                    end if;
              else
                  next_state <= WRITE_WAIT;
              end if;
    
         when WRITE_WAIT =>
                        if (((row_address_conflict = '1') or (bank_address_conflict = '1')) or
                              (auto_ref = '1' and wait_cycles = "0000")) then 
                              -- or (wait_cycles = "0000") ) then
                                next_state <= precharge_after_rd_wr;
                        elsif (controller_wrdata_fifo_rden_r = '1' and wrdata_fifo_empty = '0') then 
                            if wrburst_end_cnt = "010" then  
                                next_state <= burst_write;
                            elsif wrburst_end_cnt = "001" then 
                                next_state <= FIRST_WRITE;
                            else
                                next_state <= write_wait;
                            end if;
                         else
                            if (rdaddr_fifo_empty = '0') then
                                next_state <= read_after_write;
                            else
                                 next_state <= write_wait;
                            end if;
                         end if;
                                                   
          when BURST_WRITE =>
                     if burst_length = "001" then
                         if (wrdata_fifo_empty = '0' and controller_wrdata_fifo_rden_r = '1')  then 
                            next_state <= BURST_WRITE;
                          else
                            next_state <= WRITE_WAIT;
                          end if;   
                     else
                         next_state <= WRITE_WAIT;
                     end if; 
                     
          when WRITE_AFTER_READ =>
                     if ((row_address_conflict = '1') or (bank_address_conflict = '1')) then
                            next_state <= precharge_after_rd_wr;
                     elsif (controller_wrdata_fifo_rden_r = '1') then  
               
                          next_state <= FIRST_WRITE;
                     else
                          next_state <= WRITE_AFTER_READ;
                     end if;          
         
          when READ_AFTER_WRITE =>
                     if (wrburst_end_cnt = "000" and rdaddr_fifo_empty = '0') then 
                         next_state <= read_wait;
                     else
                          next_state <= read_after_write;
                     end if;
                        
          when FIRST_READ =>
                     if burst_length = "001" then
                        if (controller_rdaddr_fifo_rden_3r = '1' and controller_rdaddr_fifo_rden_2r = '1') then
                           next_state <= burst_read;
                        else
                           next_state <= read_wait;
                        end if;
                     else
                        next_state <= read_wait;
                     end if;
                     
          when READ_WAIT =>
                     if (((row_address_conflict = '1') or (bank_address_conflict = '1')) or
                              (auto_ref = '1' and wait_cycles = "0000"))then
                                  -- or(wait_cycles = "0000")) then
                            next_state <= precharge_after_rd_wr;
                     elsif ((burst_length = "011" and controller_rdaddr_fifo_rden_4r = '1') or
                            (burst_length = "001" and controller_rdaddr_fifo_rden_3r = '1')or
                             (burst_length = "010" and controller_rdaddr_fifo_rden_2r = '1')) then
                              if (RDBURST_END_CNT1(1 downto 0) = "01") then
                                 next_state <= burst_read;
                              elsif (RDBURST_END_CNT1(1 downto 0) = "00") then
                                 next_state <= FIRST_READ;
                              end if;
                     elsif controller_wraddr_fifo_rden_2r = '1' then
                            next_state <= WRITE_AFTER_READ;
                     elsif burst_terminate = '1' then
                            next_state <= burst_stop;
                     else
                            next_state <= read_wait;
                     end if;
                                                                
          when BURST_READ =>
                     if burst_length = "001" then
                        if (controller_rdaddr_fifo_rden_3r = '1' and controller_rdaddr_fifo_rden_2r = '1') then
                            next_state <= BURST_READ;
                        else
                            next_state <= READ_WAIT;
                        end if;
                     else
                         next_state <= READ_WAIT;
                     end if; 
                    
          when BURST_STOP =>
                    next_state <= READ_WAIT;
                    
          when precharge_after_rd_wr =>
                     if RAS_COUNT = "0000" then
                        next_state <= PRECHARGE;
                     else
                        next_state <= precharge_after_rd_wr;
                     end if;
          when others =>
                next_state <= IDLE;
        end case;
 end if;
end process;    

--************************************************************************************************
-- address generation logic
--************************************************************************************************

address_config(11 downto 7) <= "00000" when INIT_COUNT = "010" else 
                               "00010" when INIT_COUNT = "011" else
                               "01000" when current_state = PRECHARGE else 
                               "00000";

address_config(6 downto 4) <=  "000" when INIT_COUNT = "010" else
                               cas_latency when current_state = LOAD_MODE_REG else
                                "000";
address_config(3)          <= '0' ; -- design uses sequential burst
address_config(2 downto 0) <=  "000" when INIT_COUNT = "010" else
                               burst_length when current_state = LOAD_MODE_REG else
                              "000";

ddr_address1 <=    address_config when current_state = LOAD_MODE_REG or current_state = PRECHARGE else
                   ROW_ADDRESS_reg when current_state = ACTIVE else
                   ("0000" & COLUMN_ADDRESS_reg4) when ((current_state = BURST_WRITE or current_state = FIRST_WRITE)
                                                      and burst_length = "001") else
                   ("0000" & COLUMN_ADDRESS_reg3) when (current_state = BURST_WRITE or current_state = FIRST_WRITE)
                                                       else                                  
                   ("0000" & COLUMN_ADDRESS_reg3) when ((current_state = BURST_READ or current_state = FIRST_READ) 
                                                                    and burst_length = "001") else
                   ("0000" & COLUMN_ADDRESS_reg1) when (current_state = BURST_READ or current_state = FIRST_READ) else
                   "000000000000";

    
    
     DDR_BA1 <=     "01" when  (current_state = LOAD_MODE_REG and INIT_COUNT = "010")  else
                    bank_ADDRESS_reg when ( current_state = ACTIVE or current_state = FIRST_WRITE or
                                          current_state = BURST_WRITE or current_state = BURST_READ
                                          or current_state = FIRST_READ)else
                    "00";

--********************************************************************************************************
--  register row address
--********************************************************************************************************
-- during the first read/write burst, the active address is not set. hence row address set is 0
-- after the first ACTIVE command has been issued, row-address-set = 1. 
-- the row address conflict logic starts to check for conflicts only after row_address_set has been set to 1
process(clk180, rst)
begin
if rst = '1' then
   row_address_active_reg <= "000000000000";
   row_address_conflict <= '0';
   row_address_set_1 <= '0';
   row_address_set   <= '1';
elsif clk180'event and clk180 = '1' then
   if next_state = ACTIVE then
       row_address_active_reg <= row_address_reg;
       row_address_set_1 <= '1';
   else
       row_address_active_reg <= row_address_active_reg;
       row_address_set_1 <= row_address_set_1;
   end if;
   row_address_set <= row_address_set_1;
   if ((row_address /= row_address_active_reg) and row_address_set = '1') then
       row_address_conflict <= '1';
   else
       row_address_conflict <= '0';
   end if;

end if;   

end process;

--********************************************************************************************************
--  register bank address
--********************************************************************************************************

process(clk180, rst)
begin
if rst = '1' then
   bank_address_active <= "00";
   bank_address_conflict <= '0';
   bank_address_set_1 <= '0';
   bank_address_set <= '0';
elsif clk180'event and clk180 = '1' then
   if next_state = ACTIVE then
       bank_address_active <= bank_address_reg;
       bank_address_set_1 <= '1';
   else
       bank_address_active <= bank_address_active;
       bank_address_set_1 <= bank_address_set_1;
   end if;
   bank_address_set <= bank_address_set_1 ;
   
   if ((bank_address /= bank_address_active) and bank_address_set = '1') then
       bank_address_conflict <= '1'; 
   else
       bank_address_conflict <= '0';
   end if;
  
end if;   

end process;

--********************************************************************************************************
--  register column address
--********************************************************************************************************
process(clk180, rst)
begin
  if rst = '1' then
     column_address_reg1 <= "00000000";
     column_address_reg2 <= "00000000";
     column_address_reg3 <= "00000000";
     column_address_reg4 <= "00000000"; 
     column_address_reg5 <= "00000000";
     column_address_reg6 <= "00000000";  
 elsif clk180'event and clk180 = '1' then
     if ((row_address_conflict = '0' and bank_address_conflict = '0')) then   
          column_address_reg1 <= column_address;                        
          column_address_reg2 <= column_address_reg1;
          column_address_reg3 <= column_address_reg2; 
          column_address_reg4 <= column_address_reg3; 
          column_address_reg5 <= column_address_reg4;
          column_address_reg6 <= column_address_reg5;  
     else
          column_address_reg1 <= column_address_reg1;                         
          column_address_reg2 <= column_address_reg2;                        
          column_address_reg3 <= column_address_reg3;   
          column_address_reg4 <= column_address_reg4;  
          column_address_reg5 <= column_address_reg5;
          column_address_reg6 <= column_address_reg6;  
     end if;                     
        
 end if;
end process;

--**************************************************************************************************
--Pipeline stages for ddr_address and ddr_ba
--**************************************************************************************************

process(clk180,rst)
begin 
if rst = '1' then
   ddr_address2  <= "000000000000";
   ddr_address3  <= "000000000000";
   ddr_ba2       <= "00";
   ddr_ba3       <= "00";
elsif clk180'event and clk180 = '1' then
    ddr_address2 <= ddr_address1;
    ddr_address3 <= ddr_address2;
    ddr_ba2      <= ddr_ba1;
    ddr_ba3      <= ddr_ba2;
end if;
end process;

process(clk180, rst)
begin
 if rst = '1' then
   ddr_address4  <= "000000000000";
   ddr_address   <= "000000000000";
 elsif clk180'event and clk180 = '1' then
   ddr_address4  <= ddr_address3;
   ddr_address   <= ddr_address4;
 end if;
end process;

process(clk180, rst)
begin
  if rst = '1' then
     ddr_ba4 <= "00";
     ddr_ba5 <= "00";
     ddr_ba <= "ZZ";
  elsif clk180'event and clk180 = '1' then
     ddr_ba4 <= ddr_ba3;--ba3
     ddr_ba5 <= ddr_ba4;
     ddr_ba  <= ddr_ba4; 
  end if;
end process;

--************************************************************************************************
-- control signals to the Memory
--************************************************************************************************

ddr_rasb1 <= '0' when ( current_state = ACTIVE or current_state = PRECHARGE or current_state = AUTO_REFRESH 
                    or current_state = LOAD_MODE_REG) else
         '1';

ddr_casb1 <= '0' when ( current_state = BURST_READ or current_state = FIRST_READ or current_state = BURST_WRITE 
                        or current_state = FIRST_WRITE or current_state = AUTO_REFRESH or 
                        current_state = LOAD_MODE_REG) else
         '1';

ddr_web1  <= '0' when (current_state = BURST_WRITE or current_state = FIRST_WRITE 
                         or current_state = BURST_STOP 
                         or current_state = PRECHARGE or current_state = LOAD_MODE_REG) else
         '1';

--*************************************************************************************************
-- register CONTROL SIGNALS outputs
--**************************************************************************************************
process(clk180, rst)
begin
if rst = '1'  then
   ddr_rasb3 <= '1';
   ddr_casb3 <= '1';
   ddr_web3  <= '1';
   ddr_rasb2 <= '1';
   ddr_casb2 <= '1';
   ddr_web2  <= '1';

elsif clk180'event and clk180 = '1' then
   ddr_rasb2    <= ddr_rasb1;
   ddr_casb2    <= ddr_casb1;
   ddr_web2     <= ddr_web1;
   ddr_rasb3     <= ddr_rasb2;
   ddr_casb3    <= ddr_casb2;
   ddr_web3     <= ddr_web2;
end if;
end process;

process(clk180, rst)
begin
if rst = '1' then
   ddr_rasb4  <= '1';
   ddr_casb4  <= '1';
   ddr_web4   <= '1';
   ddr_rasb   <= '1';
   ddr_casb   <= '1';
   ddr_web    <= '1';
elsif clk180'event and clk180 = '1' then
   ddr_rasb4  <= ddr_rasb3;
   ddr_casb4  <= ddr_casb3;
   ddr_web4   <= ddr_web3;
   ddr_rasb   <= ddr_rasb4;
   ddr_casb   <= ddr_casb4;
   ddr_web    <= ddr_web4;
end if; 
end process;
  
                             
end arc_controller;                
         
 




   