--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       data_path.vhd
--
--  Description :     This module forms the data path through the IOB and the DDR memory device
--                    This module handles generating data to the DDR SDRAM during a write
--                    and handling data sent from the DDR SDRAM during a READ
--
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity data_path is
port(
     user_input_data    : in std_logic_vector(71 downto 0);
     clk0                : in std_logic;
     reset              : in std_logic;
     cntrl_data_val     : in std_logic;
     write_enable       : in std_logic;
     read_enable        : in std_logic;
     burst_length       : in std_logic_vector(2 downto 0);
     rclk               : in std_logic;
     wclk               : in std_logic;
     u_data_val         : out std_logic;
     ddr_dq              : inout std_logic_vector(31 downto 0);
     ddr_dm              : out std_logic_vector(3 downto 0);
     dqs                 : in std_logic_vector(3 downto 0);
     controller_output_data   : out std_logic_vector(63 downto 0));
end data_path;

architecture arc_data_path of data_path is

component sync_dqs2clk
port(
    input_data_rising  : in std_logic_vector(31 downto 0);
    input_data_falling : in std_logic_vector(31 downto 0);
    input_data_val     : in std_logic;                     -- input data valid
    rclk               : in std_logic;                     -- recapture clock
    data_val_in        : in std_logic;                     -- signal from controller indicating valid data
    output_data        : out std_logic_vector(63 downto 0); 
    clk0               : in std_logic;                     -- output clock
    output_data_val    : out std_logic;
    reset              : in std_logic);
end component;

component v2_ddr_iob_8 
port(
     read_data_rising    : out std_logic_vector(7 downto 0);   -- Receive data, captured on rising edge
     read_data_falling   : out std_logic_vector(7 downto 0);   -- Receive data, captured on falling edge
     ddr_data            : inout std_logic_vector(7 downto 0); -- Bi-directional DDR SDRAM data bus
     write_data_falling  : in std_logic_vector(7 downto 0);    -- Transmit data, output on falling edge
     write_data_rising   : in std_logic_vector(7 downto 0);    -- Transmit data, output on rising edge
     wclk                : in std_logic;                       -- Clock input for write
     dqs                 : in std_logic;                       -- eceive data clock
     write_en_val        : in std_logic;
     read_en_val         : in std_logic;
     reset               : in std_logic);
end component;

component v2_dm_iob 
port (
      write_data_mask_rising : in std_logic;
      write_data_mask_falling : in std_logic;
      wclk                    : in std_logic;
      reset                   : in std_logic;
      write_en_val           : in std_logic;
      ddr_dm                 : out std_logic);
      
end component;
signal ctrl_data_val1         : std_logic;
signal ctrl_data_val_out      : std_logic;
signal output_data_val        : std_logic;
signal read_data_rising       : std_logic_vector(31 downto 0);
signal read_data_falling      : std_logic_vector(31 downto 0);
signal write_data_rising      : std_logic_vector(31 downto 0);
signal write_data_rising1      : std_logic_vector(31 downto 0);
signal write_data_rising2      : std_logic_vector(31 downto 0);
signal write_data_rising3      : std_logic_vector(31 downto 0);
signal write_data_rising4      : std_logic_vector(31 downto 0);
signal write_data_rising_int  : std_logic_vector(31 downto 0);
signal write_data_falling     : std_logic_vector(31 downto 0);
signal write_data_falling1     : std_logic_vector(31 downto 0);
signal write_data_falling2     : std_logic_vector(31 downto 0);
signal write_data_falling3     : std_logic_vector(31 downto 0);
signal write_data_falling4     : std_logic_vector(31 downto 0);
signal write_data_falling5     : std_logic_vector(31 downto 0);
signal write_data_falling_int : std_logic_vector(31 downto 0);
signal write_data_mask_rising      : std_logic_vector(3 downto 0);
signal write_data_mask_rising1      : std_logic_vector(3 downto 0);
signal write_data_mask_rising2      : std_logic_vector(3 downto 0);
signal write_data_mask_rising3      : std_logic_vector(3 downto 0);
signal write_data_mask_rising4      : std_logic_vector(3 downto 0);
signal write_data_mask_rising_int  : std_logic_vector(3 downto 0);
signal write_data_mask_falling     : std_logic_vector(3 downto 0);
signal write_data_mask_falling1     : std_logic_vector(3 downto 0);
signal write_data_mask_falling2     : std_logic_vector(3 downto 0);
signal write_data_mask_falling3     : std_logic_vector(3 downto 0);
signal write_data_mask_falling4     : std_logic_vector(3 downto 0);
signal write_data_mask_falling5     : std_logic_vector(3 downto 0);
signal write_data_mask_falling_int : std_logic_vector(3 downto 0);
signal u_data_i_P1            : std_logic_vector(63 downto 0);
signal u_data_i_P2            : std_logic_vector(63 downto 0);
signal u_data_i_val           : std_logic_vector(63 downto 0);
signal write_en_P1            : std_logic; -- write enable Pipeline stage
signal write_en_P2            : std_logic;
signal write_en_P3            : std_logic;
signal write_en_val           : std_logic; -- write enable value
signal write_en_val_1         : std_logic;
signal write_en_int           : std_logic;
signal read_en_P1             : std_logic; -- read enable pipeline stage
signal read_en_P2             : std_logic;
signal read_en_P3             : std_logic;
signal read_en_P4             : std_logic;
signal read_en_P5             : std_logic;
signal read_en_P6             : std_logic;
signal read_en_val            : std_logic; -- read enable register
signal read_en_int            : std_logic;
signal data_val_in            : std_logic;
signal write_val              : std_logic;
signal write_val_int          : std_logic;

 begin

 
process(wclk,reset)
begin
   if reset = '1' then
       write_data_rising_int   <= (others => '0');
       write_data_rising1      <= (others => '0');
       write_data_rising2      <= (others => '0');
       write_data_rising3      <= (others => '0');
       write_data_rising4      <= (others => '0');
       write_data_rising       <= (others => '0');
 
   elsif wclk'event and wclk = '0' then
       
       write_data_rising_int   <= user_input_data(71 downto 40);
       write_data_rising1      <= write_data_rising_int;
       write_data_rising2      <= write_data_rising1;
       write_data_rising3      <= write_data_rising2;
       write_data_rising4      <= write_data_rising3;
       write_data_rising       <= write_data_rising4;
       
   end if;
end process;

process(wclk, reset)
begin
    if reset = '1' then
       write_data_falling_int  <= (others => '0');
       write_data_falling1     <= (others => '0');
       write_data_falling2     <= (others => '0');
       write_data_falling3     <= (others => '0');
       write_data_falling4     <= (others => '0');
       write_data_falling5      <= (others => '0');
       write_data_falling      <= (others => '0');
    elsif wclk'event and wclk = '1' then
       write_data_falling_int  <= user_input_data(39 downto 8);
       write_data_falling1     <= write_data_falling_int;
       write_data_falling2     <= write_data_falling1;
       write_data_falling3     <= write_data_falling2;
       write_data_falling4     <= write_data_falling3;
       write_data_falling5      <= write_data_falling4;
       write_data_falling      <= write_data_falling5;
    end if;
end process;

process(wclk,reset)
begin
   if reset = '1' then
       write_data_mask_rising_int   <= (others => '0');
       write_data_mask_rising1      <= (others => '0');
       write_data_mask_rising2      <= (others => '0');
       write_data_mask_rising3      <= (others => '0');
       write_data_mask_rising4      <= (others => '0');
       write_data_mask_rising       <= (others => '0');
 
   elsif wclk'event and wclk = '0' then
       
       write_data_mask_rising_int   <= user_input_data(7 downto 4);
       write_data_mask_rising1      <= write_data_mask_rising_int;
       write_data_mask_rising2      <= write_data_mask_rising1;
       write_data_mask_rising3      <= write_data_mask_rising2;
       write_data_mask_rising4      <= write_data_mask_rising3;
       write_data_mask_rising       <= write_data_mask_rising4;
       
   end if;
end process;

process(wclk, reset)
begin
    if reset = '1' then
       write_data_mask_falling_int  <= (others => '0');
       write_data_mask_falling1     <= (others => '0');
       write_data_mask_falling2     <= (others => '0');
       write_data_mask_falling3     <= (others => '0');
       write_data_mask_falling4     <= (others => '0');
       write_data_mask_falling5      <= (others => '0');
       write_data_mask_falling      <= (others => '0');
    elsif wclk'event and wclk = '1' then
       write_data_mask_falling_int  <= user_input_data(3 downto 0);
       write_data_mask_falling1     <= write_data_mask_falling_int;
       write_data_mask_falling2     <= write_data_mask_falling1;
       write_data_mask_falling3     <= write_data_mask_falling2;
       write_data_mask_falling4     <= write_data_mask_falling3;
       write_data_mask_falling5      <= write_data_mask_falling4;
       write_data_mask_falling      <= write_data_mask_falling5;
    end if;
end process;


-- Data path for read enable
process(wclk, reset)
begin
 if reset = '1' then
     read_en_P1 <= '0';
     read_en_P2 <= '0';
    
 elsif wclk'event and wclk = '1' then
     read_en_P1 <= read_enable;
     read_en_P2 <= read_en_P1;
 end if;
end process;

process(wclk,reset)
begin
  if reset = '1' then
     read_en_P3 <= '0';
     read_en_P4 <= '0';
  elsif wclk'event and wclk =  '0' then
     read_en_P3 <= read_en_P2;
     read_en_P4 <= read_en_P3;
      
  end if;
end process;


process(wclk, reset)
begin
  if reset = '1' then
     read_en_P5 <= '0';
     read_en_val <= '0';
  elsif wclk'event and wclk = '1' then
     read_en_P5 <= read_en_P4;
     read_en_val <= read_en_P5;
      
  end if;
end process;

--read_en_val <= read_en_P5 when burst_length = "001" else
--               read_en_P6;


-- data path for write enable
process(wclk,reset)
begin
 if reset = '1' then
    write_en_P1 <= '0';
    write_en_P2 <= '0';
    write_en_P3 <= '0';
 elsif wclk'event and wclk = '1' then
     write_en_P1 <= write_enable;
     write_en_P2 <= write_en_P1;
     write_en_P3 <= write_en_P2;
     
 end if;
end process;

-- data path for write enable
process(wclk,reset)
begin
 if reset = '1' then
    write_en_int <= '0';
    write_en_val <= '0';
 elsif wclk'event and wclk = '0' then
     write_en_int <= write_en_P2;
     write_en_val <= write_en_int;
 end if;
end process;


process(clk0)
begin
  if clk0'event and clk0 = '1' then
     ctrl_data_val1 <= cntrl_data_val;
     ctrl_data_val_out <= ctrl_data_val1;
     write_val_int <= read_en_P2;
     write_val     <= write_val_int;
  end if;
end process;

 
--***********************************************************************
--    Clock domain transfer
--     Received data needs to be transferred from the DQS clock domain back 
--     to the system clock domain. This is done through the sync_dqs2clk 
--     synchronizer module.
--
--     This module implements the clock domain transfer through a dual-port
--     LUT RAM. All control signals are double-registered on the recapture
--     (rclk) domain to remove any possibility of metastability violations.
--     Once written into the dual-port LUT RAM, data is immediately read
--     out on the system clock domain on u_data_o. A logic high on u_data_val
--     indicates this read data is valid.
--**********************************************************************

  SYNC : sync_dqs2clk port map
       (
       input_data_rising   => read_data_rising(31 downto 0),
       input_data_falling  => read_data_falling(31 downto 0),
       input_data_val      => write_val,  
       rclk                => rclk, 
       data_val_in         => ctrl_data_val_out,
       output_data         => controller_output_data(63 downto 0), 
       clk0                => clk0, 
       output_data_val     => u_data_val, 
       reset                 => reset);
            
--***********************************************************************\
--     DDR IOB Instantiations
--     This section controls the width of the accessed DDR SDRAM bus, and must
--     be manually changed for different widths. Instantiate a 'v2_ddr_iob_8'
--     module for each 8 DDR bits required (i.e. for an 8 bit bus, instantiate
--     1 v2_ddr_iob_8 module).  
--     
--     
--***********************************************************************/

  U_BYTE0 : v2_ddr_iob_8 port map 
     (
     read_data_rising    => read_data_rising(7 downto 0), 
     read_data_falling   => read_data_falling(7 downto 0),
     ddr_data            => ddr_dq(7 downto 0),
     write_data_falling  => write_data_falling(7 downto 0), 
     write_data_rising   => write_data_rising(7 downto 0), 
     wclk                => wclk, 
     dqs                 => dqs(0), 
     write_en_val        => write_en_val, 
     read_en_val         => read_en_val, 
     reset               => reset);
     
  
  U_BYTE1 : v2_ddr_iob_8 port map 
     (
     read_data_rising    => read_data_rising(15 downto 8), 
     read_data_falling   => read_data_falling(15 downto 8),
     ddr_data            => ddr_dq(15 downto 8),
     write_data_falling  => write_data_falling(15 downto 8), 
     write_data_rising   => write_data_rising(15 downto 8), 
     wclk                => wclk,
     dqs                 => dqs(1), 
     write_en_val        => write_en_val, 
     read_en_val         => read_en_val, 
     reset               => reset);
  
 U_BYTE2 : v2_ddr_iob_8 port map 
     (
     read_data_rising    => read_data_rising(23 downto 16), 
     read_data_falling   => read_data_falling(23 downto 16),
     ddr_data            => ddr_dq(23 downto 16),
     write_data_falling  => write_data_falling(23 downto 16), 
     write_data_rising   => write_data_rising(23 downto 16), 
     wclk                => wclk,
     dqs                 => dqs(2), 
     write_en_val        => write_en_val, 
     read_en_val         => read_en_val, 
     reset               => reset);
  
 U_BYTE3 : v2_ddr_iob_8 port map 
     (
     read_data_rising    => read_data_rising(31 downto 24), 
     read_data_falling   => read_data_falling(31 downto 24),
     ddr_data            => ddr_dq(31 downto 24),
     write_data_falling  => write_data_falling(31 downto 24), 
     write_data_rising   => write_data_rising(31 downto 24), 
     wclk                => wclk,
     dqs                 => dqs(3), 
     write_en_val        => write_en_val, 
     read_en_val         => read_en_val, 
     reset               => reset);

 U_DMASK_byte0 : v2_dm_iob port map
     (
      write_data_mask_rising => write_data_mask_rising(0),
      write_data_mask_falling => write_data_mask_falling(0),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm                  => ddr_dm(0));
 
 U_DMASK_byte1 : v2_dm_iob port map
     (
      write_data_mask_rising => write_data_mask_rising(1),
      write_data_mask_falling => write_data_mask_falling(1),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm                  => ddr_dm(1));
      
 U_DMASK_byte2 : v2_dm_iob port map
     (
      write_data_mask_rising => write_data_mask_rising(2),
      write_data_mask_falling => write_data_mask_falling(2),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm                  => ddr_dm(2));
      
 U_DMASK_byte : v2_dm_iob port map
     (
      write_data_mask_rising => write_data_mask_rising(3),
      write_data_mask_falling => write_data_mask_falling(3),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm                  => ddr_dm(3));
 
end arc_data_path;




