library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--library synplify; 
--use synplify.attributes.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity v2_dm_iob is
port (
      write_data_mask_rising : in std_logic;
      write_data_mask_falling : in std_logic;
      wclk                    : in std_logic;
      reset                   : in std_logic;
      write_en_val           : in std_logic;
      ddr_dm                 : out std_logic);
end v2_dm_iob;
                             
architecture arc_v2_dm_iob of v2_dm_iob is

component FDDRRSE 
port( Q  : out std_logic;
      C0 : in std_logic;
      C1 : in std_logic;
      CE : in std_logic;
      D0 : in std_logic;
      D1 : in std_logic;
      R  : in std_logic;
      S  : in std_logic);
end component;

component FDRE
port(
       D  : in std_logic;
       R  : in std_logic; 
       C  : in std_logic; 
       Q  : out std_logic; 
       CE : in std_logic);
end component;
  
component FDCE 
port(
       D   : in std_logic; 
       CLR : in std_logic; 
       C   : in std_logic; 
       Q   : out std_logic; 
       CE  : in std_logic);
end component;

component OBUFT
port (
       I : in std_logic;
       T : in std_logic;
       O : out std_logic);
end component;

      

--***********************************************************************\
--     Internal signal declaration
--***********************************************************************/
  
signal ddr_en       : std_logic;  -- Tri-state enable signal
signal ddr_dm_q     : std_logic;  -- Data output intermediate signal
signal wclkb      : std_logic;
signal GND          : std_logic;
signal clock_en     : std_logic := '1';
signal enable_b     : std_logic;
 
-- synplicity attribute 
--attribute xc_props of DQ_RISE: label is "IOB=TRUE"; 
--attribute xc_props of DQ_FALL: label is "IOB=TRUE"; 

begin


wclkb <= not wclk;
GND    <= '0';
enable_b <= not write_en_val;

-- Output Transmission data path

DDR_OUT : FDDRRSE port map
            (Q  => ddr_dm_q, 
             C0 => wclk, 
             C1 => wclkb, 
             CE => clock_en,
             D0 => write_data_mask_rising, 
             D1 => write_data_mask_falling, 
             R  => GND, 
             S  => GND);
   
DQ_T   :   FDCE port map
           ( D   => enable_b, 
             CLR => reset, 
             C   => wclk, 
             Q   => ddr_en, 
             CE  => clock_en);

  
DQ_OBUFT : OBUFT port map
             ( I => ddr_dm_q,
               T => ddr_en,
               O => ddr_dm);

end arc_v2_dm_iob;
