--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       v2_dqs_iob.vhd
--
--  Description :     This module generates a single DQS IOB. This includes a DDR output
--                    flip-flop for the DQS signal, and an input buffer to distribute the 
--                    DQS line. 
--                    
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--

entity v2_dqs_iob is
port(
     clk   : in std_logic;
     clk180 : in std_logic;
     DDR_DQS_reset : in std_logic;
     DDR_DQS_enable : in std_logic;
     DDR_DQS        : inout std_logic;
     DQS            : out std_logic);
end v2_dqs_iob;
 
     
architecture arc_V2_dqs_iob of V2_dqs_iob is
component FD
port( D : in std_logic;
      Q : out std_logic;
      C : in std_logic);
end component;

component FDDRRSE 
port( Q : out std_logic;
      C0 : in std_logic;
      C1 : in std_logic;
      CE : in std_logic;
      D0 : in std_logic;
      D1 : in std_logic;
      R  : in std_logic;
      S  : in std_logic);
end component;

component OBUFT
port(
      I : in std_logic;
      T : in std_logic;
      O : out std_logic);
end component;

component IBUF_SSTL2_II
port(
      I : in std_logic;
      O : out std_logic);
end component;

signal dqs_q     : std_logic;
signal DDR_DQS_enable1  : std_logic;
signal VCC       : std_logic;
signal GND       : std_logic;
signal clk_b     : std_logic;


begin

--***********************************************************************
--     Output DDR generation
--     This includes instantiation of the output DDR flip flop.
--     Additionally, to keep synthesis tools from register sharing, manually
--     instantiate the output tri-state flip-flop.
--*********************************************************************** 
VCC <= '1';
GND <= '0';

U1 : FD port map  ( D => DDR_DQS_enable,
                    Q => DDR_DQS_enable1,
                    C => clk);

U2 : FDDRRSE port map (  Q => DQS_q,
                        C0 => clk,
                        C1 => clk180,
                        CE => VCC,
                        D0 => VCC,
                        D1 => GND,
                         R => DDR_DQS_reset,
                         S => GND);
--***********************************************************************
--    IO buffer for DQS signal. Allows for distribution of DQS
--     to the data (DQ) loads.
--***********************************************************************
U3 : OBUFT  port map ( I => DQS_q,
                       T => DDR_DQS_enable1 ,
                       O => DDR_DQS);

U4 : IBUF_SSTL2_II port map ( I => DDR_DQS,
                              O => DQS);


  

end arc_V2_dqs_iob;