--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       clk_dcm.vhd
--
--  Description :     This module generates the system clock for controller block
--                    This also generates the recapture clock, clock for the
--                    Refresh counter and also for the data path
--
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity clk_dcm is 
port(
   input_clk  : in std_logic;
   rst        : in std_logic;
   ddr_clk    : out std_logic;
   ddr_clkb   : out std_logic;
   clk        : out std_logic;
   clk180     : out std_logic;
   rclk       : out std_logic;
   WCLK       : out std_logic;
   DCM_LOCK   : out std_logic;
   clk_dv16   : out std_logic);
end clk_dcm;

architecture arc_clk_dcm of clk_dcm is

component DCM
-- pragma translate_off
    generic ( 
             DLL_FREQUENCY_MODE : string := "LOW";
             DUTY_CYCLE_CORRECTION : boolean := TRUE;
             CLKOUT_PHASE_SHIFT    : string := "FIXED";
             PHASE_SHIFT           : integer ; 
             CLKDV_DIVIDE : real := 16.0
             
            );  
-- pragma translate_on

    port ( CLKIN     : in  std_logic;
           CLKFB     : in  std_logic;
           DSSEN     : in  std_logic;
           PSINCDEC  : in  std_logic;
           PSEN      : in  std_logic;
           PSCLK     : in  std_logic;
           RST       : in  std_logic;
           CLK0      : out std_logic;
           CLK90     : out std_logic;
           CLK180    : out std_logic;
           CLK270    : out std_logic;
           CLK2X     : out std_logic;
           CLK2X180  : out std_logic;
           CLKDV     : out std_logic;
           CLKFX     : out std_logic;
           CLKFX180  : out std_logic;
           LOCKED    : out std_logic;
           PSDONE    : out std_logic;
           STATUS    : out std_logic_vector(7 downto 0)
          );
end component;

 component IBUFG
  port ( I : in std_logic;
         O : out std_logic);
 end component;

 component BUFG
  port ( I : in std_logic;
         O : out std_logic);
 end component;

 component FDDRRSE 
   port( Q : out std_logic;
        C0 : in std_logic;
        C1 : in std_logic;
        CE : in std_logic;
        D0 : in std_logic;
        D1 : in std_logic;
         R : in std_logic;
         S : in std_logic);
end component;

signal input_clk_ibuf      : std_logic;
signal CLK0_o              : std_logic;
signal CLK0_buf            : std_logic;
signal CLK180_o            : std_logic;
signal CLK180_buf          : std_logic;
signal VCC                 : std_logic;
signal GND                 : std_logic;
signal CLK_b               : std_logic;
signal DCM1_LOCK           : std_logic;
signal DCM2_LOCk           : std_logic;
signal DCM3_LOCK           : std_logic;
signal RCLK_buf            : std_logic;
signal RCLK_o              : std_logic;
signal clk_dv16_o          : std_logic;
signal clk_dv16_obuf       : std_logic;
signal clk0_3              : std_logic;
signal clk0_3_buf          : std_logic;
signal WCLK_o              : std_logic;
signal WCLK_buf            : std_logic;
signal DCM_WCLK_lock       : std_logic;

-- Atrtributes for Recapture clock DCM

attribute CLKOUT_PHASE_SHIFT : string;
attribute PHASE_SHIFT        : integer;
attribute DLL_FREQUENCY_MODE : string; 
attribute DUTY_CYCLE_CORRECTION : string;
attribute CLKDV_DIVIDE : string; 

attribute DLL_FREQUENCY_MODE of DCM_INST1    : label is "LOW";
attribute DUTY_CYCLE_CORRECTION of DCM_INST1 : label is "TRUE";
attribute CLKDV_DIVIDE of DCM_INST1          : label is "16.0";

attribute CLKOUT_PHASE_SHIFT of DCM_WCLK     : label is "FIXED";
attribute PHASE_SHIFT of DCM_WCLK            : label is 230; 

attribute CLKOUT_PHASE_SHIFT of DCM_INST2    : label is "FIXED";
attribute PHASE_SHIFT of DCM_INST2           : label is 140; 
attribute DLL_FREQUENCY_MODE of DCM_INST2    : label is "LOW";
attribute DUTY_CYCLE_CORRECTION of DCM_INST2 : label is "TRUE";

 

begin

VCC <= '1';
GND <= '0';

CLK <= CLK0_buf;
CLK180 <= CLK180_buf;
RCLK <= RCLK_buf;
WCLK <= WCLK_buf;

CLK_b <= not CLK0_buf;

CLK_INST1 : IBUFG port map ( I => input_clk,
                             O => input_clk_ibuf);
 
DCM_INST1 :  DCM  generic map ( phase_shift => 0)
                  port map ( CLKIN => input_clk_ibuf,
                            CLKFB => CLK0_buf,
                            DSSEN => GND,
                            PSINCDEC => GND,
                            PSEN => GND,
                            PSCLK => GND,
                            RST => RST,
                            CLK0 => CLK0_o,
                            CLK90 => open,
                            CLK180 => CLK180_o,
                            CLK270 => open,
		            CLK2X => open,
                            CLK2X180 => open,
                            CLKDV => clk_dv16_o,
                            CLKFX => open,
                            CLKFX180 => open,
                            LOCKED => DCM1_LOCK,
                            PSDONE => open,
                            STATUS => open);

BUFG_CLK0   : BUFG port map ( I => CLK0_o ,
                               O => CLK0_buf);
                               
BUFG_CLK180   : BUFG port map ( I => CLK180_o ,
                               O => CLK180_buf);                               
 
BUFG_CLKDV   : BUFG port map ( I => clk_dv16_o,
                               O => clk_dv16);  


DDRCLK_INST  : FDDRRSE port map  ( Q  => DDR_CLK, 
                                   C0 => CLK0_buf, 
                                   C1 => CLK180_buf, 
                                   CE => VCC, 
                                   D0 => VCC, 
                                   D1 => GND, 
                                   R  => GND, 
                                   S  => GND);

DDRCLKB_INST  : FDDRRSE port map  (Q  => DDR_CLKB, 
                                   C0 => CLK0_buf, 
                                   C1 => CLK180_buf, 
                                   CE => VCC, 
                                   D0 => GND, 
                                   D1 => VCC, 
                                   R  => GND, 
                                   S  => GND);
--***********************************************************************\
--     DCM_WCLK Component Instantiation
--     This clock is a phase-shifted version of input clock. This allows designers
--     to adjust the position of the data with respect to DQS to gain more
--     margin at the memory during a WRITE
--     The data and dqs need to be center aligned at the memory
--     Since DQS goes through the trace delay specified in the appnote,
--     the clock for the data is phase shifted, so that data falls in the 
--     center of the DQS at the memory
--     This DCM is optional, designers may choose to use CLK0 instead to 
--     clock the data during a WRITE, Careful timing analysis is required
--     Check the timing analysis section on the appnote
--
--     calculation of phase shift
--     negative phase shift of 0.5 ns = positive phase shift of 4.5 ns
--     PS value = (4.5/5)*256 = 230
--***********************************************************************/

DCM_WCLK :  DCM  generic map (PHASE_SHIFT => 230)

                 port map (  CLKIN => input_clk_ibuf,
                            CLKFB => WCLK_buf,
                            DSSEN => GND,
                            PSINCDEC => GND,
                            PSEN => GND,
                            PSCLK => GND,
                            RST => RST,
                            CLK0 => WCLK_o,
                            CLK90 => open,
                            CLK180 => open,
                            CLK270 => open,
			    CLK2X => open,
                            CLK2X180 => open,
                            CLKDV => open,
                            CLKFX => open,
                            CLKFX180 => open,
                            LOCKED => DCM_WCLK_LOCK,
                            PSDONE => open,
                            STATUS => open);

BUFG_WCLK : BUFG port map   ( I=> WCLK_o,
                             O => WCLK_buf);


--***********************************************************************\
--     DCM_RCLK Component Instantiation
--     This clock is a phase-shifted version of u_clk. This allows designers
--     to adjust the position of recapture clock which captures data after
--     the DQS signal.
--***********************************************************************/

  

DCM_INST2 :  DCM  generic map ( phase_shift => 80)
                  port map ( CLKIN => input_clk_ibuf,
                            CLKFB => RCLK_buf,
                            DSSEN => GND,
                            PSINCDEC => GND,
                            PSEN => GND,
                            PSCLK => GND,
                            RST => RST,
                            CLK0 => RCLK_o,
                            CLK90 => open,
                            CLK180 => open,
                            CLK270 => open,
			    CLK2X  => open,
                            CLK2X180 => open,
                            CLKDV => open,
                            CLKFX => open,
                            CLKFX180 => open,
                            LOCKED => DCM2_LOCK,
                            PSDONE => open,
                            STATUS => open);

 
BUFG_INST3 : BUFG port map (I => RCLK_o, 
                            O => RCLK_buf);

  
DCM_LOCK <= DCM1_LOCK and DCM2_LOCK and DCM_WCLK_LOCK;

end arc_clk_dcm;