--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       ddr_controller_top
--
--  Description :     
--                    Main DDR SDRAM controller block. This includes the following
--                    features:
--                    - The main controller state machine that controlls the 
--                    initialization process upon power up, as well as the 
--                    read, write, and refresh commands. 
--                    - handles the data path during READ and WRITEs.
--                    - Generates control signals for other modules, including the
--                      data strobe(DQS) signal
--
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
--library synplify; 
--use synplify.attributes.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
library XilinxCoreLib;   
-- pragma translate_on
--

entity DDR_CONTROLLER_TOP is
port(
     user_input_clk      : in std_logic;
     user_input_data     : in std_logic_vector(71 downto 0);
     user_output_data    : out std_logic_vector(63 downto 0);
     user_data_valid     : out std_logic;
     user_input_address  : in std_logic_vector(21 downto 0);
     user_config_register     : in std_logic_vector(9 downto 0);
     user_command_register    : in std_logic_vector(2 downto 0);
     user_wrdata_fifo_wren    : in std_logic;
     user_wraddr_fifo_wren    : in std_logic;
     user_rdaddr_fifo_wren    : in std_logic;
     user_rddata_fifo_rden    : in std_logic;
     wrdata_fifo_full         : out std_logic;
     wraddr_fifo_full         : out std_logic;
     rdaddr_fifo_full         : out std_logic;
     rddata_fifo_empty        : out std_logic;
     user_rst                 : in std_logic;
     ddr_clk             : out std_logic ;
     ddr_clkb            : out std_logic;
     ddr_dqs             : inout std_logic_vector(3 downto 0):= (OTHERS => 'Z');
     ddr_dq              : inout std_logic_vector(31 downto 0):= (OTHERS => 'Z');
     ddr_cke             : out std_logic;
     ddr_csb             : out std_logic;
     ddr_rasb            : out std_logic;
     ddr_casb            : out std_logic;
     ddr_web             : out std_logic;
     ddr_dm              : out std_logic_vector(3 downto 0);
     ddr_BA              : out std_logic_vector(1 downto 0);
     ddr_address         : out std_logic_vector(11 downto 0));
end DDR_CONTROLLER_TOP;


architecture ARC_DDR_CONTROLLER_TOP of DDR_CONTROLLER_TOP is

component controller 
  port(
    clk180           : in std_logic;
    dcm_lock         : in std_logic;
    clk_dv16         : in std_logic;
    rst              : in std_logic;
    read_address     : in std_logic_vector(21 downto 0);
    write_address    : in std_logic_vector(21 downto 0);
    config_register  : in std_logic_vector(9 downto 0);
    command_register : in std_logic_vector(2 downto 0);
    wrdata_fifo_empty : in std_logic;
    wraddr_fifo_empty : in std_logic;
    rdaddr_fifo_empty : in std_logic;
    controller_wrdata_fifo_rden : out std_logic;
    controller_wraddr_fifo_rden : out std_logic;
    controller_rdaddr_fifo_rden : out std_logic;
    burst_length_val : out std_logic_vector(2 downto 0);
    ddr_rasb         : out std_logic;
    ddr_casb         : out std_logic;
    ddr_web          : out std_logic;
    ddr_BA           : out std_logic_vector(1 downto 0);
    ddr_address      : out std_logic_vector(11 downto 0);
    ddr_cke          : out std_logic;
    ddr_csb          : out std_logic;
    dqs_enable       : out std_logic;
    dqs_reset        : out std_logic;
    write_enable     : out std_logic;
    read_enable      : out std_logic;
    cntrl_data_val   : out std_logic);
end component;

component DQS_strobe
  port (
      DQS_enable : in std_logic;
      DQS_reset  : in std_logic;
      clk        : in std_logic;
      clk180     : in std_logic;
      DDR_DQS    : inout std_logic_vector(3 downto 0);
      DQS        : out std_logic_vector(3 downto 0));
end component;

component data_path
   port (
     user_input_data    : in std_logic_vector(71 downto 0);
     clk0                : in std_logic;
     reset              : in std_logic;
     cntrl_data_val     : in std_logic;
     write_enable       : in std_logic;
     read_enable        : in std_logic;
     burst_length       : in std_logic_vector(2 downto 0);
     rclk               : in std_logic;
     wclk               : in std_logic;
     u_data_val         : out std_logic;
     ddr_dq              : inout std_logic_vector(31 downto 0);
     ddr_dm            : out std_logic_vector(3 downto 0);
     dqs                 : in std_logic_vector(3 downto 0);
     controller_output_data   : out std_logic_vector(63 downto 0));
end component;

component clk_dcm 
  port(
   input_clk  : in std_logic;
   rst        : in std_logic;
   ddr_clk    : out std_logic;
   ddr_clkb   : out std_logic;
   clk        : out std_logic;
   clk180     : out std_logic;
   rclk       : out std_logic;
   wclk       : out std_logic;
   DCM_LOCK   : out std_logic;
   clk_dv16   : out std_logic);
end component;

component wrdata_fifo
	port (
	clk: IN std_logic;
	sinit: IN std_logic;
	din: IN std_logic_VECTOR(71 downto 0);
	wr_en: IN std_logic;
	rd_en: IN std_logic;
	dout: OUT std_logic_VECTOR(71 downto 0);
	full: OUT std_logic;
	empty: OUT std_logic);
end component;

component wraddr_fifo
	port (
	clk: IN std_logic;
	sinit: IN std_logic;
	din: IN std_logic_VECTOR(21 downto 0);
	wr_en: IN std_logic;
	rd_en: IN std_logic;
	dout: OUT std_logic_VECTOR(21 downto 0);
	full: OUT std_logic;
	empty: OUT std_logic);
end component;

component rdaddr_fifo
	port (
	clk: IN std_logic;
	sinit: IN std_logic;
	din: IN std_logic_VECTOR(21 downto 0);
	wr_en: IN std_logic;
	rd_en: IN std_logic;
	dout: OUT std_logic_VECTOR(21 downto 0);
	full: OUT std_logic;
	empty: OUT std_logic);
end component;

component rddata_fifo
	port (
	clk: IN std_logic;
	sinit: IN std_logic;
	din: IN std_logic_VECTOR(63 downto 0);
	wr_en: IN std_logic;
	rd_en: IN std_logic;
	dout: OUT std_logic_VECTOR(63 downto 0);
	full: OUT std_logic;
	empty: OUT std_logic);
end component;

signal clk0_int       : std_logic;
signal clk90_int     : std_logic;
signal clk_dv16       : std_logic;
signal clk180_int     : std_logic;
signal rclk           : std_logic;
signal wclk_int       : std_logic;
signal dcm_lock       : std_logic;
signal dqs_enable     : std_logic;
signal dqs_reset      : std_logic;
signal write_enable   : std_logic;
signal read_enable    : std_logic;
signal cntrl_data_val : std_logic;
signal dqs            : std_logic_vector(3 downto 0);
signal sys_rst_o      : std_logic := '1';
signal sys_rst_1      : std_logic := '1';
signal sys_rst        : std_logic := '1';
signal burst_length   : std_logic_vector(2 downto 0);
signal controller_input_address : std_logic_vector(21 downto 0);
signal controller_input_data : std_logic_vector(71 downto 0);
signal wrdata_fifo_empty : std_logic;
signal wraddr_fifo_empty : std_logic;
signal rdaddr_fifo_empty : std_logic;
signal controller_wrdata_fifo_rden : std_logic;
signal controller_wraddr_fifo_rden : std_logic;
signal controller_rddata_fifo_wren : std_logic;
signal controller_rdaddr_fifo_rden : std_logic;
signal user_data_val : std_logic;
signal controller_output_data : std_logic_vector(63 downto 0);
signal controller_write_address : std_logic_vector(21 downto 0);
signal controller_read_address : std_logic_vector(21 downto 0);
signal rddata_fifo_full : std_logic;
signal user_data_valid1 : std_logic;
signal user_data_valid2 : std_logic;
signal user_data_valid3 : std_logic;
signal user_output_data1 : std_logic_vector(63 downto 0);
signal user_input_data_reg   : std_logic_vector(71 downto 0);
signal user_input_address_reg  : std_logic_vector(21 downto 0);
signal user_wrdata_fifo_wren_reg : std_logic;
signal user_wraddr_fifo_wren_reg : std_logic;
signal user_rdaddr_fifo_wren_reg : std_logic;
signal user_rddata_fifo_rden_reg : std_logic;
signal user_config_reg  : std_logic_vector(9 downto 0);
signal user_command_reg : std_logic_vector(2 downto 0);
signal controller_output_data_val : std_logic;

begin

controller_rddata_fifo_wren <= user_data_val;

process(clk0_int)
begin
   sys_rst_1 <= sys_rst_o;
   sys_rst   <= sys_rst_1;
end process; 

-- synchronise RESET
process(clk0_int)
begin
  if user_rst = '1' or DCM_LOCK = '0' then
      sys_rst_o <= '1' ;
  else
      sys_rst_o <= '0';
  end if;
end process;


process(clk0_int)
begin
   if sys_rst = '1' then
      user_data_valid1 <= '0';
      user_data_valid2 <= '0';
      user_data_valid3 <= '0';
      user_data_valid <= '0';
   elsif clk0_int'event and clk0_int = '1' then
      user_data_valid1 <= user_data_val;  
      user_data_valid2 <= user_data_valid1;
      user_data_valid3 <= user_data_valid2;
      user_data_valid <= user_data_valid3;
   end if;
end process;

process(clk0_int, sys_rst)
begin
   if sys_rst = '1' then
      user_output_data <= (OTHERS => '0');
      user_rddata_fifo_rden_reg   <= '0';
   elsif clk0_int'event and clk0_int = '1' then
      user_output_data <= user_output_data1;
      user_rddata_fifo_rden_reg    <= user_rddata_fifo_rden;
   end if;
end process;


process(clk180_int, sys_rst)
begin
   if sys_rst = '1' then   
       user_input_address_reg      <= (others => '0');
       user_wrdata_fifo_wren_reg   <= '0';
       user_wraddr_fifo_wren_reg   <= '0';
       user_rdaddr_fifo_wren_reg   <= '0';
       user_config_reg             <= (others => '0');
       user_command_reg            <= (others => '0');
       user_input_data_reg  <= (others => '0');
   elsif clk180_int'event and clk180_int = '1' then
       user_input_address_reg       <= user_input_address;
       user_wrdata_fifo_wren_reg    <= user_wrdata_fifo_wren;
       user_wraddr_fifo_wren_reg    <= user_wraddr_fifo_wren;
       user_rdaddr_fifo_wren_reg    <= user_rdaddr_fifo_wren;
       user_config_reg              <= user_config_register;           
       user_command_reg             <= user_command_register;
       user_input_data_reg          <= user_input_data;  
    end if;
end process;       
   
      



INST_1 : controller port map (clk180             => clk180_int,
                              dcm_lock           => dcm_lock,   
                              clk_dv16           => clk_dv16,
                              rst                => sys_rst,
                              read_address       => controller_read_address,
                              write_address      => controller_write_address,
                              config_register    => user_config_reg,
                              command_register   => user_command_reg,
                              wrdata_fifo_empty  => wrdata_fifo_empty,
                              wraddr_fifo_empty  => wraddr_fifo_empty,
                              rdaddr_fifo_empty  => rdaddr_fifo_empty,
                              controller_wrdata_fifo_rden => controller_wrdata_fifo_rden,
                              controller_wraddr_fifo_rden => controller_wraddr_fifo_rden,
                              controller_rdaddr_fifo_rden => controller_rdaddr_fifo_rden,
                              burst_length_val   => burst_length ,
                              ddr_rasb           => ddr_rasb,
                              ddr_casb           => ddr_casb,
                              ddr_web            => ddr_web,
                              ddr_BA             => ddr_BA,
                              ddr_address        => ddr_address,
                              ddr_cke            => ddr_cke,
                              ddr_csb            => ddr_csb,
                              dqs_enable         => dqs_enable,
                              dqs_reset          => dqs_reset,
                              write_enable       => write_enable,
                              read_enable        => read_enable,
                              cntrl_data_val     => cntrl_data_val);

INST_2 : dqs_strobe port map (DQS_enable => DQS_enable,
                               DQS_reset  => DQS_reset,
                               clk        => clk0_int,
                               clk180     => clk180_int,
                               DDR_DQS    => DDR_DQS,
                               DQS        => dqs);

INST_3 : data_path port map ( user_input_data  => controller_input_data(71 downto 0),
                              clk0              => clk0_int,
                              reset            => sys_rst,
                              cntrl_data_val   => cntrl_data_val,
                              write_enable     => write_enable,
                              read_enable      => read_enable,
                              burst_length     => burst_length,
                              rclk             => rclk,
                              wclk             => wclk_int,
                              u_data_val       => user_data_val,
                              ddr_dq           => ddr_dq,
                              ddr_dm           => ddr_dm,
                              dqs              => dqs,
                              controller_output_data => controller_output_data);

INST_4 : clk_dcm port map   ( input_clk => user_input_clk,
                              rst       => user_rst,
                              ddr_clk   => ddr_clk,
                              ddr_clkb  => ddr_clkb,
                              clk       => clk0_int,
                              clk180    => clk180_int,
                              rclk      => rclk,
                              wclk      => wclk_int,
                              DCM_LOCK  => dcm_lock,
                              clk_dv16  => clk_dv16);
                              
                              
INST_wrdata : wrdata_fifo port map (
			      clk                => clk180_int,
			      sinit              => sys_rst,
			      din                => user_input_data_reg,
			      wr_en              => user_wrdata_fifo_wren_reg,
			      rd_en              => controller_wrdata_fifo_rden,
			      dout               => controller_input_data,
			      full               => wrdata_fifo_full,
			      empty              => wrdata_fifo_empty);

INST_wraddr : wraddr_fifo port map (
			      clk                => clk180_int,
			      sinit              => sys_rst,
			      din                => user_input_address_reg,
			      wr_en              => user_wraddr_fifo_wren_reg,
			      rd_en              => controller_wraddr_fifo_rden,
			      dout               => controller_write_address,
			      full               => wraddr_fifo_full,
			      empty              => wraddr_fifo_empty);

INST_rddata : rddata_fifo port map (
			      clk                => clk0_int,
			      sinit              => sys_rst,
			      din                => controller_output_data,
			      wr_en              => controller_rddata_fifo_wren,
			      rd_en              => user_rddata_fifo_rden_reg,
			      dout               => user_output_data1,
			      full               => rddata_fifo_full,
			      empty              => rddata_fifo_empty);

INST_rdaddr : rdaddr_fifo port map (
			      clk                => clk180_int,
			      sinit              => sys_rst,
			      din                => user_input_address_reg,
			      wr_en              => user_rdaddr_fifo_wren_reg,
			      rd_en              => controller_rdaddr_fifo_rden,
			      dout               => controller_read_address,
			      full               => rdaddr_fifo_full,
			      empty              => rdaddr_fifo_empty);

end ARC_DDR_CONTROLLER_TOP;
                                                    