--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       ddr_controller_top_tb.vhd
--
--  Description :     This module is the test bench for the top level controller 
--                    module
--                    
--  Date - Revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--

entity tb_ddr_controller is
end tb_ddr_controller;

architecture arc_tb_ddr_controller of tb_ddr_controller is

component MT46V4M32
  PORT (
        Dq    : INOUT STD_LOGIC_VECTOR (32 - 1 DOWNTO 0) := (OTHERS => 'Z');
        Dqs   : INOUT STD_LOGIC_VECTOR (3 DOWNTO 0) := "ZZZZ";
        Addr  : IN    STD_LOGIC_VECTOR (12 - 1 DOWNTO 0);
        Ba    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0);
        Clk   : IN    STD_LOGIC;
        Clk_n : IN    STD_LOGIC;
        Cke   : IN    STD_LOGIC;
        Cs_n  : IN    STD_LOGIC;
        Ras_n : IN    STD_LOGIC;
        Cas_n : IN    STD_LOGIC;
        We_n  : IN    STD_LOGIC;
        Dm    : IN    STD_LOGIC_VECTOR (3 DOWNTO 0));
end component;

component DDR_CONTROLLER_TOP 
port(
     user_input_clk      : in std_logic;
     user_input_data     : in std_logic_vector(71 downto 0);
     user_output_data    : out std_logic_vector(63 downto 0):=(OTHERS => 'Z');
     user_data_valid     : out std_logic;
     user_input_address  : in std_logic_vector(21 downto 0);
     user_config_register     : in std_logic_vector(9 downto 0);
     user_command_register    : in std_logic_vector(2 downto 0);
     user_wrdata_fifo_wren    : in std_logic;
     user_wraddr_fifo_wren    : in std_logic;
     user_rdaddr_fifo_wren    : in std_logic;
     user_rddata_fifo_rden    : in std_logic;
     wrdata_fifo_full         : out std_logic;
     wraddr_fifo_full         : out std_logic;
     rdaddr_fifo_full         : out std_logic;
     rddata_fifo_empty        : out std_logic;
     user_rst                 : in std_logic;
     ddr_clk             : out std_logic ;
     ddr_clkb            : out std_logic;
     ddr_dqs             : inout std_logic_vector(3 downto 0):= (OTHERS => 'Z');
     ddr_dq              : inout std_logic_vector(31 downto 0):= (OTHERS => 'Z');
     ddr_cke             : out std_logic;
     ddr_csb             : out std_logic;
     ddr_rasb            : out std_logic;
     ddr_casb            : out std_logic;
     ddr_web             : out std_logic;
     ddr_dm              : out std_logic_vector(3 downto 0);
     ddr_BA              : out std_logic_vector(1 downto 0);
     ddr_address         : out std_logic_vector(11 downto 0));
end component;

CONSTANT clock_period    : TIME := 5.0 ns;
signal half_clk_period   : TIME := 2.5 ns;
signal clk               : std_logic;
signal data_in           : std_logic_vector(71 downto 0);
--signal data_mask_in      : std_logic_vector(7 downto 0);
signal data_out          : std_logic_vector(63 downto 0);
signal data_val          : std_logic;
signal address_in        : std_logic_vector(21 downto 0);
--signal bank_address      : std_logic_vector(1  downto 0);
signal config_register   : std_logic_vector(9 downto 0);
signal command_register  : std_logic_vector(2 downto 0);
signal rst               : std_logic;
signal ddr_clk           : std_logic;
signal ddr_clkb          : std_logic;
signal ddr_dqs_fpga      : std_logic_vector(3  downto 0);
signal ddr_dqs_sdram     : std_logic_vector(3  downto 0);
signal ddr_dq            : std_logic_vector(31 downto 0);
signal ddr_cke           : std_logic;
signal ddr_csb           : std_logic;
signal ddr_rasb          : std_logic;
signal ddr_casb          : std_logic;
signal ddr_web           : std_logic;
signal ddr_dm            : std_logic_vector(3  downto 0);
signal ddr_BA            : std_logic_vector(1  downto 0);
signal ddr_address       : std_logic_vector(11 downto 0); 
signal CMD               : std_logic_vector(2 downto 0);
signal enable_o          : std_logic;
--signal enable_P1          : std_logic;
--signal enable_P2          : std_logic;
--signal enable_P3          : std_logic;
--signal enable           : std_logic := '0';
signal user_wrdata_fifo_wren : std_logic;
signal user_wraddr_fifo_wren : std_logic;
signal user_rddata_fifo_rden : std_logic;  
signal user_rdaddr_fifo_wren : std_logic;  
signal wrdata_fifo_full      : std_logic;
signal wraddr_fifo_full      : std_logic;
signal rddata_fifo_empty      : std_logic;
signal rdaddr_fifo_full      : std_logic;

--signal num_xfers         : std_logic_vector(3  downto 0);

begin

CMD <= ( ddr_rasb & ddr_casb & ddr_web );

user_rddata_fifo_rden <= '1' when rddata_fifo_empty = '0' else
                         '0';

process(clk)
begin
  if clk'event and clk = '1' then
    if CMD = "100" then -- Write
       enable_o <= '0';
    elsif CMD = "101" then -- Read
       enable_o <= '1';
    else
       enable_o <= enable_o;
    end if;
  end if;
end process;

-- During a READ
ddr_dqs_fpga <= ddr_dqs_sdram after 0.75 ns when enable_o = '1' else
                "ZZZZ";

-- During a WRITE
ddr_dqs_sdram <= ddr_dqs_fpga after 0.75 ns when enable_o = '0' else
                 "ZZZZ";


UUT_1 : DDR_CONTROLLER_TOP port map ( user_input_clk     => clk,
                                      user_input_data    => data_in,
                                      user_output_data   => data_out,
                                      user_data_valid    => data_val,  
                                      user_input_address => address_in,  
                                      user_config_register    => config_register, 
                                      user_command_register   => command_register,
                                      user_wrdata_fifo_wren   => user_wrdata_fifo_wren,
                                      user_wraddr_fifo_wren   => user_wraddr_fifo_wren,
                                      user_rdaddr_fifo_wren   => user_rdaddr_fifo_wren,      
                                      user_rddata_fifo_rden   => user_rddata_fifo_rden ,
                                      wrdata_fifo_full        => wrdata_fifo_full ,
                                      wraddr_fifo_full        => wraddr_fifo_full,
                                      rdaddr_fifo_full        => rdaddr_fifo_full,
                                      rddata_fifo_empty        => rddata_fifo_empty ,  
                                      user_rst           => rst,  
                                      ddr_clk            => ddr_clk,  
                                      ddr_clkb           => ddr_clkb,  
                                      ddr_dqs            => ddr_dqs_fpga, 
                                      ddr_dq             => ddr_dq, 
                                      ddr_cke            => ddr_cke,  
                                      ddr_csb            => ddr_csb, 
                                      ddr_rasb           => ddr_rasb,
                                      ddr_casb           => ddr_casb,  
                                      ddr_web            => ddr_web, 
                                      ddr_dm             => ddr_dm,  
                                      ddr_BA             => ddr_BA,  
                                      ddr_address        => ddr_address);

UUT_2 :  MT46V4M32        port map  ( Dq                => ddr_dq,
                                      Dqs               => ddr_dqs_sdram,
                                      Addr              => ddr_address,
                                      Ba                => ddr_BA,
                                      Clk               => ddr_clk,
                                      Clk_n             => ddr_clkb,
                                      Cke               => ddr_cke,
                                      Cs_n              => ddr_csb,
                                      Ras_n             => ddr_rasb,
                                      Cas_n             => ddr_casb,
                                      We_n              => ddr_web,
                                      Dm                => ddr_dm);


-- generate clock for simulation

process
begin
  clk <= '1';
  
  LOOP
     WAIT FOR 2500 ps;
     clk <= NOT clk;
  end LOOP;
end process;

-- RESET
P_RST: process
begin
	RST <= '1';
	wait for 160 ns;
	RST <= '0';
	wait;
end process P_RST; 

-- start simulation
-- 
process
begin
data_in <= "000000000000000000000000000000000000000000000000000000000000000000000000";
address_in <= "0000000000000000000000";
config_register <= "0000000000";
command_register <= "000";
user_wrdata_fifo_wren <= '0';
user_wraddr_fifo_wren <= '0';
--user_rddata_fifo_rden <= '0';
user_rdaddr_fifo_wren <= '0';    

                                                             
wait for 150 ns;  -- DCM to LOCK                                 

--*****************************************************************************
--  Memory initialization
--***************************************************************************** 

config_register <= "0000110010"; -- select cas = 3, burstlength = 010 -2
command_register <= "000";

--*****************************************************************************
-- wait for memory Initialization to complete
--*****************************************************************************
wait for 400 ns;

--*****************************************************************************
-- 200 clock cycles are required between initialization and issuing a READ cmd
--*****************************************************************************
wait for 1000 ns;

--**************************************************************************
-- Issue a Write Burst of 4 
-- Row Address: "000011111111" COlumn Address : "00001000"  BA : "00
--**************************************************************************
user_wrdata_fifo_wren <= '1';
user_wraddr_fifo_wren <= '1';
address_in <= "0000111111110000100001";
data_in <= "000000000000000000000000000010100000000000000000000000000000101100000000";--0000000A0000000B
wait for clock_period;

user_wrdata_fifo_wren <= '1';
user_wraddr_fifo_wren <= '0';
data_in <= "000000000000000000000000000011000000000000000000000000000000110100000000";--0000000C0000000D
wait for clock_period;

user_wraddr_fifo_wren <= '0';
user_wrdata_fifo_wren <= '0';
wait for 7*clock_period;

address_in <= "0000111111110000100001";
user_rdaddr_fifo_wren <= '1';
wait for clock_period;

user_rdaddr_fifo_wren <= '0';

wait for 20*clock_period;

--******************************************************************************
-- check consecutive WRITEs
-- Issue a Write Burst of 2 
-- Row Address: "000011100000" COlumn Address : "00001000"  BA : "00
--******************************************************************************
-- row address =>0E0 ; column address =>00 ; BA => 2
address_in <= "0000111000000000000010"; 
data_in <= "000000000000000000000000000000010000000000000000000000000000001000000000";--0000000100000002
user_wraddr_fifo_wren <= '1';
user_wrdata_fifo_wren <= '1';
wait for clock_period;

data_in <= "000000000000000000000000000000110000000000000000000000000000010000000000";--0000000300000004
user_wraddr_fifo_wren <= '1';
user_wrdata_fifo_wren <= '1';
wait for clock_period;

-- row address =>0E0 ; column address =>08 ; BA => 2 
address_in <= "0000111000000000100010";
data_in <= "000000000000000000000000000001010000000000000000000000000000011000000000";--0000000500000006
user_wrdata_fifo_wren <= '1';
user_wraddr_fifo_wren <= '1';
wait for  clock_period;

data_in <= "000000000000000000000000000001110000000000000000000000000000100000000000";--0000000700000008
user_wraddr_fifo_wren <= '1';
user_wrdata_fifo_wren <= '1';
wait for clock_period;

-- row address =>0E0 ; column address =>10 ; BA => 2
address_in <= "0000111000000001000010";
data_in <= "000000000000000000000000000010010000000000000000000000000000101000000000";--000000090000000A
user_wrdata_fifo_wren <= '1';
user_wraddr_fifo_wren <= '1';
wait for  clock_period;

data_in <= "000000000000000000000000000010110000000000000000000000000000110000000000";--0000000B0000000C
user_wraddr_fifo_wren <= '1';
user_wrdata_fifo_wren <= '1';
wait for clock_period;

-- row address =>0E0 ; column address =>18 ; BA => 2
address_in <= "0000111000000001100010";
user_wrdata_fifo_wren <= '1';
user_wraddr_fifo_wren <= '1';
data_in <= "000000000000000000000000000011010000000000000000000000000000111000000000";--0000000D0000000E
wait for  clock_period;

data_in <= "000000000000000000000000000011110000000000000000000000000001000000000000";--0000000F00000010
user_wraddr_fifo_wren <= '1';
user_wrdata_fifo_wren <= '1';
wait for clock_period;

user_wraddr_fifo_wren <= '0';
user_wrdata_fifo_wren <= '0';
command_register <= "000";
wait for 20*clock_period;

--******************************************************************************
-- check consecutive READs
-- 
-- Row Address: "000011111000" COlumn Address : "00101000"  BA : "00
--****************************************************************************** 
address_in <= "0000111000000000000010";
user_rdaddr_fifo_wren <= '1';
wait for clock_period;

user_rdaddr_fifo_wren <= '0';
wait for clock_period;

address_in <= "0000111000000000100010";
user_rdaddr_fifo_wren <= '1';
wait for clock_period;

user_rdaddr_fifo_wren <= '0';
wait for clock_period;

address_in <= "0000111000000001000010";
user_rdaddr_fifo_wren <= '1';
wait for clock_period;

user_rdaddr_fifo_wren <= '0';
wait for clock_period;

address_in <= "0000111000000001100010";
user_rdaddr_fifo_wren <= '1';
wait for clock_period;

user_rdaddr_fifo_wren <= '0';
wait for 11*clock_period;

----******************************************************************************
---- check writes following reads
---- 
---- Row Address: "000011111000" COlumn Address : "00101000"  BA : "10
----****************************************************************************** 
-- row address =>0E0 ; column address =>00 ; BA => 2
address_in <= "0000111000000000000010"; 
data_in <= "000000000000000000000000000000010000000000000000000000000000001000000000";--0000000100000002
user_wraddr_fifo_wren <= '1';
user_wrdata_fifo_wren <= '1';
command_register <= "000";
wait for clock_period;

data_in <= "000000000000000000000000000000110000000000000000000000000000010000000000";--0000000300000004
user_wraddr_fifo_wren <= '1';
user_wrdata_fifo_wren <= '1';
wait for clock_period;

-- row address =>0E0 ; column address =>08 ; BA => 2 
address_in <= "0000111000000000100010";
data_in <= "000000000000000000000000000001010000000000000000000000000000011000000000";--0000000500000006
user_wrdata_fifo_wren <= '1';
user_wraddr_fifo_wren <= '1';
wait for  clock_period;

data_in <= "000000000000000000000000000001110000000000000000000000000000100000000000";--0000000700000008
user_wraddr_fifo_wren <= '1';
user_wrdata_fifo_wren <= '1';
wait for clock_period;

-- row address =>0E0 ; column address =>10 ; BA => 2
address_in <= "0000111000000001000010";
data_in <= "000000000000000000000000000010010000000000000000000000000000101000000000";--000000090000000A
user_wrdata_fifo_wren <= '1';
user_wraddr_fifo_wren <= '1';
wait for  clock_period;

data_in <= "000000000000000000000000000010110000000000000000000000000000110000000000";--0000000B0000000C
user_wraddr_fifo_wren <= '1';
user_wrdata_fifo_wren <= '1';
wait for clock_period;

-- row address =>0E0 ; column address =>18 ; BA => 2
address_in <= "0000111000000001100010";
user_wrdata_fifo_wren <= '1';
user_wraddr_fifo_wren <= '1';
data_in <= "000000000000000000000000000011010000000000000000000000000000111000000000";--0000000D0000000E
wait for  clock_period;

data_in <= "000000000000000000000000000011110000000000000000000000000001000000000000";--0000000F00000010
user_wraddr_fifo_wren <= '1';
user_wrdata_fifo_wren <= '1';
wait for clock_period;

user_wraddr_fifo_wren <= '0';
user_wrdata_fifo_wren <= '0';
command_register <= "000";
wait for 20*clock_period;

--******************************************************************************
-- check consecutive READs
-- 
-- Row Address: "000011111000" COlumn Address : "00101000"  BA : "00
--****************************************************************************** 
address_in <= "0000111000000000000010";
user_rdaddr_fifo_wren <= '1';
wait for clock_period;

user_rdaddr_fifo_wren <= '0';
wait for clock_period;

address_in <= "0000111000000000100010";
user_rdaddr_fifo_wren <= '1';
wait for clock_period;

user_rdaddr_fifo_wren <= '0';
wait for clock_period;

address_in <= "0000111000000001000010";
user_rdaddr_fifo_wren <= '1';
wait for clock_period;

user_rdaddr_fifo_wren <= '0';
wait for clock_period;

address_in <= "0000111000000001100010";
user_rdaddr_fifo_wren <= '1';
wait for clock_period;

user_rdaddr_fifo_wren <= '0';
wait for 11*clock_period;

end process;
end arc_tb_ddr_controller;
















                                      