LIBRARY ieee;
    USE ieee.std_logic_1164.ALL;

ENTITY tb IS
END tb;

ARCHITECTURE test OF tb IS

    CONSTANT tCK       : TIME    :=  7.500 ns;
    CONSTANT addr_bits : INTEGER := 12;
    CONSTANT data_bits : INTEGER := 32;

    COMPONENT mt46v4m32
        PORT (
            Dq    : INOUT STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
            Dqs   : INOUT STD_LOGIC_VECTOR (3 DOWNTO 0);
            Addr  : IN    STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0);
            Ba    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0);
            Clk   : IN    STD_LOGIC;
            Clk_n : IN    STD_LOGIC;
            Cke   : IN    STD_LOGIC;
            Cs_n  : IN    STD_LOGIC;
            Cas_n : IN    STD_LOGIC;
            Ras_n : IN    STD_LOGIC;
            We_n  : IN    STD_LOGIC;
            Dm    : IN    STD_LOGIC_VECTOR (3 DOWNTO 0)
        ); 
    END COMPONENT;
  
    FOR ALL : mt46v4m32 USE ENTITY work.mt46v4m32(behave);

    SIGNAL pDq       : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
    SIGNAL pDqs      : STD_LOGIC_VECTOR (3 DOWNTO 0);
    SIGNAL pAddr     : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0) := (OTHERS => '0');
    SIGNAL pBa       : STD_LOGIC_VECTOR (1 DOWNTO 0) := "00";
    SIGNAL pClk      : STD_LOGIC := '0';
    SIGNAL pClk_n    : STD_LOGIC := '1';
    SIGNAL pCke      : STD_LOGIC := '0';
    SIGNAL pCs_n     : STD_LOGIC := '1';
    SIGNAL pRas_n    : STD_LOGIC := '1';
    SIGNAL pCas_n    : STD_LOGIC := '1';
    SIGNAL pWe_n     : STD_LOGIC := '1';
    SIGNAL pDm       : STD_LOGIC_VECTOR (3 DOWNTO 0);
    SIGNAL stim_done : boolean := false;
    SIGNAL clk_done  : boolean := false;

BEGIN
    u1: mt46v4m32
        PORT MAP(
            Dq => pDq,
            Dqs => pDqs,
            Addr => pAddr,
            Ba => pBa,
            Clk => pClk,
            Clk_n => pClk_n,
            Cke => pCke,
            Cs_n => pCs_n,
            Ras_n => pRas_n,
            Cas_n => pCas_n,
            We_n => pWe_n,
            Dm => pDm
        );

    stimulator : PROCESS

        PROCEDURE ACTIVE (Ba_in : STD_LOGIC_VECTOR (1 DOWNTO 0);
                          Addr_in : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0)) IS
        BEGIN
            pCke   <= '1';
            pCs_n  <= '0';
            pRas_n <= '0';
            pCas_n <= '1';
            pWe_n  <= '1';
            pDm    <= "0000";
            pBa    <= Ba_in;
            pAddr  <= Addr_in;
            pDq    <= (OTHERS=>'Z');
            pDqs   <= (OTHERS=>'Z');
            WAIT FOR tCK;
        END;

        PROCEDURE AUTO_REFRESH IS
        BEGIN
            pCke   <= '1';
            pCs_n  <= '0';
            pRas_n <= '0';
            pCas_n <= '0';
            pWe_n  <= '1';
            pDm    <= "0000";
            --pBa    <= "00";
            --pAddr  <= (OTHERS => '0');
            pDq    <= (OTHERS=>'Z');
            pDqs   <= (OTHERS=>'Z');
            WAIT FOR tCK;
        END;

        PROCEDURE LOAD_MODE_REG (Ba_in : STD_LOGIC_VECTOR (1 DOWNTO 0);
                                 Addr_in : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0)) IS
        BEGIN
            pCke   <= '1';
            pCs_n  <= '0';
            pRas_n <= '0';
            pCas_n <= '0';
            pWe_n  <= '0';
            pDm    <= "0000";
            pBa    <= Ba_in;
            pAddr  <= Addr_in;
            pDq    <= (OTHERS=>'Z');
            pDqs   <= (OTHERS=>'Z');
            WAIT FOR tCK;
        END;

        PROCEDURE NOP IS
        BEGIN
            pCke   <= '1';
            pCs_n  <= '0';
            pRas_n <= '1';
            pCas_n <= '1';
            pWe_n  <= '1';
            pDm    <= "0000";
            --pBa    <= "00";
            --pAddr  <= (OTHERS => '0');
            pDq    <= (OTHERS=>'Z');
            pDqs   <= (OTHERS=>'Z');
            WAIT FOR tCK;
        END;

        PROCEDURE BURST_TERMINATE IS
        BEGIN
            pCke   <= '1';
            pCs_n  <= '0';
            pRas_n <= '1';
            pCas_n <= '1';
            pWe_n  <= '0';
            pDm    <= "0000";
            --pBa    <= "00";
            --pAddr  <= (OTHERS => '0');
            pDq    <= (OTHERS=>'Z');
            pDqs   <= (OTHERS=>'Z');
            WAIT FOR tCK;
        END;

        PROCEDURE PRECHARGE (Ba_in : STD_LOGIC_VECTOR (1 DOWNTO 0);
                  Addr_in : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0)) IS
        BEGIN
            pCke   <= '1';
            pCs_n  <= '0';
            pRas_n <= '0';
            pCas_n <= '1';
            pWe_n  <= '0';
            pDm    <= "0000";
            pBa    <= Ba_in;
            pAddr  <= Addr_in;
            pDq    <= (OTHERS=>'Z');
            pDqs   <= (OTHERS=>'Z');
            WAIT FOR tCK;
        END;

        PROCEDURE READ (Ba_in : STD_LOGIC_VECTOR (1 DOWNTO 0);
                        Addr_in : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0)) IS
        BEGIN
            pCke   <= '1';
            pCs_n  <= '0';
            pRas_n <= '1';
            pCas_n <= '0';
            pWe_n  <= '1';
            pDm    <= "0000";
            pBa    <= Ba_in;
            pAddr  <= Addr_in;
            pDq    <= (OTHERS=>'Z');
            pDqs   <= (OTHERS=>'Z');
            WAIT FOR tCK;
        END;

        -- Write Burst Length = 2
        PROCEDURE WRITE_2 (Ba_in   : STD_LOGIC_VECTOR (1 DOWNTO 0);                 -- Bank
                           Addr_in : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0);     -- Column
                           Dq0_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);     -- First Data
                           Dq1_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0)) IS  -- Last Data
        BEGIN
            pCke   <= '1';
            pCs_n  <= '0';
            pRas_n <= '1';
            pCas_n <= '0';
            pWe_n  <= '0';
            pDm    <= "0000";
            pBa    <= Ba_in;
            pAddr  <= Addr_in;
            pDq    <= (OTHERS=>'Z');
            pDqs   <= (OTHERS=>'Z');
            WAIT FOR tCK / 2; pDqs <= (OTHERS=>'0');
            WAIT FOR tCK / 2; pCas_n <= '1'; pWe_n <= '1';
            WAIT FOR tCK / 4; pDq  <= Dq0_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'1');
            WAIT FOR tCK / 4; pDq  <= Dq1_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'0');
            WAIT FOR tCK / 4; pDq  <= (OTHERS=>'Z');
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'Z');
            WAIT FOR tCK / 2;
        END;

        -- Write Burst Length = 4
        PROCEDURE WRITE_4 (Ba_in   : STD_LOGIC_VECTOR (1 DOWNTO 0);                 -- Bank
                           Addr_in : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0);     -- Column
                           Dq0_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);     -- First Data
                           Dq1_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
                           Dq2_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
                           Dq3_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0)) IS  -- Last Data
        BEGIN
            pCke   <= '1';
            pCs_n  <= '0';
            pRas_n <= '1';
            pCas_n <= '0';
            pWe_n  <= '0';
            pDm    <= "0000";
            pBa    <= Ba_in;
            pAddr  <= Addr_in;
            pDq    <= (OTHERS=>'Z');
            pDqs   <= (OTHERS=>'Z');
            WAIT FOR tCK / 2; pDqs <= (OTHERS=>'0');
            WAIT FOR tCK / 2; pCas_n <= '1'; pWe_n <= '1';
            WAIT FOR tCK / 4; pDq  <= Dq0_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'1');
            WAIT FOR tCK / 4; pDq  <= Dq1_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'0');
            WAIT FOR tCK / 4; pDq  <= Dq2_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'1');
            WAIT FOR tCK / 4; pDq  <= Dq3_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'0');
            WAIT FOR tCK / 4; pDq  <= (OTHERS=>'Z');
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'Z');
            WAIT FOR tCK / 2;
        END;

        -- Write Burst Length = 8
        PROCEDURE WRITE_8 (Ba_in   : STD_LOGIC_VECTOR (1 DOWNTO 0);                 -- Bank
                           Addr_in : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0);     -- Column
                           Dq0_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);     -- First Data
                           Dq1_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
                           Dq2_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
                           Dq3_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
                           Dq4_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
                           Dq5_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
                           Dq6_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
                           Dq7_in  : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0)) IS  -- Last Data
        BEGIN
            pCke   <= '1';
            pCs_n  <= '0';
            pRas_n <= '1';
            pCas_n <= '0';
            pWe_n  <= '0';
            pDm    <= "0000";
            pBa    <= Ba_in;
            pAddr  <= Addr_in;
            pDq    <= (OTHERS=>'Z');
            pDqs   <= (OTHERS=>'Z');
            WAIT FOR tCK / 2; pDqs <= (OTHERS=>'0');
            WAIT FOR tCK / 2; pCas_n <= '1'; pWe_n <= '1';
            WAIT FOR tCK / 4; pDq  <= Dq0_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'1');
            WAIT FOR tCK / 4; pDq  <= Dq1_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'0');
            WAIT FOR tCK / 4; pDq  <= Dq2_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'1');
            WAIT FOR tCK / 4; pDq  <= Dq3_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'0');
            WAIT FOR tCK / 4; pDq  <= Dq4_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'1');
            WAIT FOR tCK / 4; pDq  <= Dq5_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'0');
            WAIT FOR tCK / 4; pDq  <= Dq6_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'1');
            WAIT FOR tCK / 4; pDq  <= Dq7_in;
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'0');
            WAIT FOR tCK / 4; pDq  <= (OTHERS=>'Z');
            WAIT FOR tCK / 4; pDqs <= (OTHERS=>'Z');
            WAIT FOR tCK / 2;
        END;

        
    BEGIN
    --  COMMAND          BA    ADDR            DQ
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
    --  We need 200 us power up sequence.  I use 10 NOPs here as a required example.
        PRECHARGE       ("00", "010000000000");                        -- PRE ALL
        NOP;
        NOP;
        LOAD_MODE_REG   ("01", "000000000000");                        -- EMR
        NOP;
        LOAD_MODE_REG   ("00", "000100100010");                        -- LMR (CL = 2, BL = 4, DLL RESET)
        NOP;
        LOAD_MODE_REG   ("00", "000000100010");                        -- LMR (CL = 2, BL = 4, this second LMR is optional)
        NOP;
        PRECHARGE       ("00", "010000000000");                        -- PRE ALL
        NOP;
        NOP;
        AUTO_REFRESH;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        AUTO_REFRESH;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        NOP;
        
        --
        -- 200 cycles DLL
        FOR I IN 1 TO 200 LOOP
            NOP;
        END LOOP;

        --
        -- Write and Read with Auto Precharge
        --
        ACTIVE          ("00", "000000000000");                        -- ACTIVATE BANK 0 ROW 0
        NOP;
        NOP;
        WRITE_4         ("00", "010000000000", "00000000000011110000000000000000",             -- WRITE BANK 0 COL 0
                                               "00000000000011100000000000000001",
                                               "00000000000011000000000000000010",
                                               "00000000000010010000000000000011");

        ACTIVE          ("01", "000000000000");                        -- ACTIVATE BANK 1 ROW 0
        NOP;
        NOP;
        WRITE_4         ("01", "010000000000", "00000000000000000000000000000100",             -- WRITE BANK 1 COL 0
                                               "00000000000000000000000000000101",
                                               "00000000000000000000000000000110",
                                               "00000000000000000000000000000111");

        ACTIVE          ("10", "000000000000");                        -- ACTIVATE BANK 2 ROW 0
        NOP;
        NOP;
        WRITE_4         ("10", "010000000000", "00000000000000000000000000001000",             -- WRITE BANK 2 COL 0
                                               "00000000000000000000000000001001",
                                               "00000000000000000000000000001010",
                                               "00000000000000000000000000001011");

        ACTIVE          ("11", "000000000000");                        -- ACTIVATE BANK 3 ROW 0
        NOP;
        NOP;
        WRITE_4         ("11", "010000000000", "00000000000000000000000000001100",             -- WRITE BANK 3 COL 0
                                               "00000000000000000000000000001101",
                                               "00000000000000000000000000001110",
                                               "00000000000000000000000000001111");

        ACTIVE          ("00", "000000000000");                        -- ACTIVATE BANK 0 ROW 0
        NOP;
        NOP;
        READ            ("00", "010000000000");                        -- READ BANK 0 COL 0
        NOP;
        NOP;
        NOP;
        NOP;

        ACTIVE          ("01", "000000000000");                        -- ACTIVATE BANK 1 ROW 0
        NOP;
        NOP;
        READ            ("01", "010000000000");                        -- READ BANK 1 COL 0
        NOP;
        NOP;
        NOP;
        NOP;

        ACTIVE          ("10", "000000000000");                        -- ACTIVATE BANK 2 ROW 0
        NOP;
        NOP;
        READ            ("10", "010000000000");                        -- READ BANK 2 COL 0
        NOP;
        NOP;
        NOP;
        NOP;

        ACTIVE          ("11", "000000000000");                        -- ACTIVATE BANK 3 ROW 0
        NOP;
        NOP;
        READ            ("11", "010000000000");                        -- READ BANK 3 COL 0
        NOP;
        NOP;
        NOP;
        NOP;

        --
        -- Write and Read with Manual Precharge
        --
        ACTIVE          ("00", "000000000001");                        -- ACTIVATE BANK 0 ROW 1
        NOP;
        NOP;
        WRITE_4         ("00", "000000000000", "00000000000000000000000000010000",             -- WRITE BANK 0 COL 0
                                               "00000000000000000000000000010001",
                                               "00000000000000000000000000010010",
                                               "00000000000000000000000000010011");
        NOP;
        PRECHARGE       ("00", "000000000000");                        -- PRE Bank 0

        ACTIVE          ("01", "000000000001");                        -- ACTIVATE BANK 1 ROW 1
        NOP;
        NOP;
        WRITE_4         ("01", "000000000000", "00000000000000000000000000010100",             -- WRITE BANK 1 COL 0
                                               "00000000000000000000000000010101",
                                               "00000000000000000000000000010110",
                                               "00000000000000000000000000010111");
        NOP;
        PRECHARGE       ("01", "000000000000");                        -- PRE Bank 1

        ACTIVE          ("10", "000000000001");                        -- ACTIVATE BANK 2 ROW 1
        NOP;
        NOP;
        WRITE_4         ("10", "000000000000", "00000000000000000000000000011000",             -- WRITE BANK 2 COL 0
                                               "00000000000000000000000000011001",
                                               "00000000000000000000000000011010",
                                               "00000000000000000000000000011011");
        NOP;
        PRECHARGE       ("10", "000000000000");                        -- PRE Bank 2

        ACTIVE          ("11", "000000000001");                        -- ACTIVATE BANK 3 ROW 1
        NOP;
        NOP;
        NOP;
        WRITE_4         ("11", "000000000000", "00000000000000000000000000011100",             -- WRITE BANK 3 COL 0
                                               "00000000000000000000000000011101",
                                               "00000000000000000000000000011110",
                                               "00000000000000000000000000011111");
        NOP;
        PRECHARGE       ("11", "000000000000");                        -- PRE Bank 3

        ACTIVE          ("00", "000000000001");                        -- ACTIVATE BANK 0 ROW 1
        NOP;
        NOP;
        NOP;
        READ            ("00", "000000000000");                        -- READ BANK 0 COL 0
        NOP;
        PRECHARGE       ("00", "000000000000");                        -- PRE Bank 0
        NOP;
        NOP;
        NOP;
        NOP;

        ACTIVE          ("01", "000000000001");                        -- ACTIVATE BANK 1 ROW 1
        NOP;
        NOP;
        NOP;
        READ            ("01", "000000000000");                        -- READ BANK 1 COL 0
        NOP;
        PRECHARGE       ("01", "000000000000");                        -- PRE Bank 1
        NOP;
        NOP;
        NOP;

        ACTIVE          ("10", "000000000001");                        -- ACTIVATE BANK 2 ROW 1
        NOP;
        NOP;
        NOP;
        READ            ("10", "000000000000");                        -- READ BANK 2 COL 0
        NOP;
        PRECHARGE       ("10", "000000000000");                        -- PRE Bank 2
        NOP;
        NOP;
        NOP;

        ACTIVE          ("11", "000000000001");                        -- ACTIVATE BANK 3 ROW 1
        NOP;
        NOP;
        NOP;
        READ            ("11", "000000000000");                        -- READ BANK 3 COL 0
        NOP;
        PRECHARGE       ("11", "000000000000");                        -- PRE Bank 3
        NOP;
        NOP;
        NOP;

        --
        -- Alternate Bank Read with Auto Precharge
        --
        ACTIVE          ("00", "000000000001");                        -- ACTIVATE BANK 0 ROW 1
        NOP;
        NOP;
        ACTIVE          ("01", "000000000001");                        -- ACTIVATE BANK 1 ROW 1
        READ            ("00", "000000000000");                        -- READ BANK 0 COL 0

        ACTIVE          ("10", "000000000001");                        -- ACTIVATE BANK 2 ROW 1
        READ            ("01", "000000000000");                        -- READ BANK 1 COL 0

        ACTIVE          ("11", "000000000001");                        -- ACTIVATE BANK 3 ROW 1
        READ            ("10", "000000000000");                        -- READ BANK 2 COL 0

        NOP;
        READ            ("11", "000000000000");                        -- READ BANK 3 COL 0
        NOP;
        NOP;
        NOP;
        NOP;

        
        NOP;
        NOP;

        stim_done <= true;
        WAIT;
    END PROCESS;

    clock : PROCESS
    VARIABLE done_time : time;
    BEGIN
        pclk <= '0';
        pclk_n <= '1';
        WAIT for tCK/2;
            WHILE not stim_done loop
                pclk <= '1';
                pclk_n <= '0';
                WAIT for tCK/2;
                pclk <= '0';
                pclk_n <= '1';
                WAIT for tCK/2;
            END LOOP;
            done_time := now + tCK;
            WHILE now < done_time LOOP  -- one last clock to finish last command
                pclk <= '1';
                pclk_n <= '0';
                WAIT for tCK/2;
                pclk <= '0';
                pclk_n <= '1';
                WAIT for tCK/2;
            END LOOP;
            ASSERT false 
                REPORT "Suspending clock activity"
                SEVERITY note;
            clk_done <= true;
            WAIT;
    END PROCESS;

END test;

