--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       v2_ddr_iob_8.vhd
--
--  Description :      This module instantiates 8 v2_ddr_iob modules. This creates a single 
--                     8 bit SDRAM data bus. 
--
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--****************************************************************************************
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--library synplify; 
--use synplify.attributes.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity v2_ddr_iob_8 is
port(
     read_data_rising    : out std_logic_vector(7 downto 0); --  Receive data, captured on rising edge
     read_data_falling   : out std_logic_vector(7 downto 0); -- Receive data, captured on falling edge
     ddr_data            : inout std_logic_vector(7 downto 0); --Bi-directional DDR SDRAM data bus
     write_data_falling  : in std_logic_vector(7 downto 0); --Transmit data, output on falling edge
     write_data_rising   : in std_logic_vector(7 downto 0); --Transmit data, output on rising edge
     wclk                : in std_logic; --Write clock input  
     dqs                 : in std_logic; --Receive data clock
     write_en_val        : in std_logic;
     read_en_val         : in std_logic;
     reset               : in std_logic);
end v2_ddr_iob_8;

architecture arc_v2_ddr_iob_8 of v2_ddr_iob_8 is

component v2_ddr_iob
port( 
    read_data_rising   : out std_logic;
    read_data_falling  : out std_logic;
    ddr_dq_inout       : inout std_logic;
    write_data_rising  : in std_logic;
    write_data_falling : in std_logic;
    wclk               : in std_logic;
    clk_rx             : in std_logic;
    write_en_val       : in std_logic;
    read_en_val        : in std_logic;
    reset              : in std_logic);
end component;


begin


DDR_0 : v2_ddr_iob  port map (
        read_data_rising   => read_data_rising(0),
        read_data_falling  => read_data_falling(0),
        ddr_dq_inout       => ddr_data(0), 
        write_data_rising  => write_data_rising(0), 
        write_data_falling => write_data_falling(0), 
        wclk                => wclk ,
        clk_rx             => dqs, 
        write_en_val       => write_en_val, 
        read_en_val        => read_en_val, 
        reset              => reset);

DDR_1 :  v2_ddr_iob  port map (
        read_data_rising   => read_data_rising(1),
        read_data_falling  => read_data_falling(1),
        ddr_dq_inout       => ddr_data(1), 
        write_data_rising  => write_data_rising(1), 
        write_data_falling => write_data_falling(1), 
        wclk                => wclk , 
        clk_rx             => dqs, 
        write_en_val       => write_en_val, 
        read_en_val        => read_en_val, 
        reset              => reset);

DDR_2 :  v2_ddr_iob  port map (
        read_data_rising   => read_data_rising(2),
        read_data_falling  => read_data_falling(2),
        ddr_dq_inout             => ddr_data(2), 
        write_data_rising  => write_data_rising(2), 
        write_data_falling => write_data_falling(2), 
        wclk                => wclk , 
        clk_rx             => dqs, 
        write_en_val       => write_en_val, 
        read_en_val        => read_en_val, 
        reset              => reset);

DDR_3 :  v2_ddr_iob  port map (
        read_data_rising   => read_data_rising(3),
        read_data_falling  => read_data_falling(3),
        ddr_dq_inout       => ddr_data(3), 
        write_data_rising  => write_data_rising(3), 
        write_data_falling => write_data_falling(3), 
        wclk               => wclk, 
        clk_rx             => dqs, 
        write_en_val       => write_en_val, 
        read_en_val        => read_en_val, 
        reset              => reset);

DDR_4 :  v2_ddr_iob  port map (
        read_data_rising   => read_data_rising(4),
        read_data_falling  => read_data_falling(4),
        ddr_dq_inout       => ddr_data(4), 
        write_data_rising  => write_data_rising(4), 
        write_data_falling => write_data_falling(4), 
        wclk               => wclk, 
        clk_rx             => dqs, 
        write_en_val       => write_en_val, 
        read_en_val        => read_en_val, 
        reset              => reset);

DDR_5 :  v2_ddr_iob  port map (
        read_data_rising   => read_data_rising(5),
        read_data_falling  => read_data_falling(5),
        ddr_dq_inout       => ddr_data(5), 
        write_data_rising  => write_data_rising(5), 
        write_data_falling => write_data_falling(5), 
        wclk               => wclk,
        clk_rx             => dqs, 
        write_en_val       => write_en_val, 
        read_en_val        => read_en_val, 
        reset              => reset);

DDR_6 :  v2_ddr_iob  port map (
        read_data_rising   => read_data_rising(6),
        read_data_falling  => read_data_falling(6),
        ddr_dq_inout       => ddr_data(6), 
        write_data_rising  => write_data_rising(6), 
        write_data_falling => write_data_falling(6), 
        wclk               => wclk,
        clk_rx             => dqs, 
        write_en_val       => write_en_val, 
        read_en_val        => read_en_val, 
        reset              => reset);

DDR_7 :  v2_ddr_iob  port map (
        read_data_rising   => read_data_rising(7),
        read_data_falling  => read_data_falling(7),
        ddr_dq_inout       => ddr_data(7), 
        write_data_rising  => write_data_rising(7), 
        write_data_falling => write_data_falling(7), 
        wclk               => wclk, 
        clk_rx             => dqs, 
        write_en_val       => write_en_val, 
        read_en_val        => read_en_val, 
        reset              => reset);

end arc_v2_ddr_iob_8; 
